\\ Copyright 2015, 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(strictargs,1);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\ngot  %s\nwant %s",x,y)));
  return(0);
}

check_equal(got,want,name="") =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong ",name,"\n",d); quit(1),
     print("ok ",check_count," ",name));
}

\\-----------------------------------------------------------------------------
\\ Gray code endpoint

\\      *
\\      |  k=1
\\      v  end=1
\\ *--->

\\ *
\\ |       k=2
\\ v       end=I
\\  <---*
\\      |
\\      v
\\ *--->

midpoint_right(start,end) = (start+end)/2 + (-I)*(end-start)/2;
check_exact(midpoint_right(0,1+I), 1)
check_exact(midpoint_right(2*I, 1+I), I)

Gray_end(k, start, end) =
{
  if(k==0, end,
     my(m = midpoint_right(start,end));
     Gray_end(k-1, end,m));
}
vector(14,k,my(k=k-1); Gray_end(k,0,b^k))

Gray_end_bk(k) =
{
  if(k==0, 1,
     b^k + (-I)*Gray_end_bk(k-1));
}
check_exact(vector(10,k,my(k=k-1); Gray_end_bk(k)), \
            vector(10,k,my(k=k-1); Gray_end(k,0,b^k)));

Gray_end_bk(k) =
{
  (-I)^k*1 + sum(j=0,k-1, (-I)^j*b^(k-j));
}
check_exact(vector(10,k,my(k=k-1); Gray_end_bk(k)), \
            vector(10,k,my(k=k-1); Gray_end(k,0,b^k)));

Gray_end_bk(k) = (-I)^k + b * ((-I)^k - b^k) / (-I - b);
check_exact(vector(10,k,my(k=k-1); Gray_end_bk(k)), \
            vector(10,k,my(k=k-1); Gray_end(k,0,b^k)));

Gray_end_bk(k) = 1/5*( (3-I)*b^k + (2+I)*(-I)^k );
check_exact(vector(10,k,my(k=k-1); Gray_end_bk(k)), \
            vector(10,k,my(k=k-1); Gray_end(k,0,b^k)));

print("delta ",vector(14,k,my(k=k-1); Gray_end_bk(k-1) - b^k))
print("delta ",vector(14,k,my(k=k-1); norm(Gray_end_bk(k-1) - b^k)))

\\-----------------------------------------------------------------------------
\\ pen up by drawing

draw(k) =
{
  pen_location = 0;
  move_total = 0;
  draw_recurse(k, 0, b^k);
  move_total;
}
draw_recurse(k, start, end) =
{
  if(k==0,
     move(start);
     pen_location=end,
     \\
     my(m = midpoint_right(start,end));
     draw_recurse(k-1, start, m);
     draw_recurse(k-1, end, m));
}
draw_line(start,end) =
{
  pen_state = 0;
  move(start);
  pen_location = end;
}
move(to) =
{
  move_total += abs(to - pen_location);
  pen_location = to;
}
vector(14,k,my(k=k-1); draw(k))

Gray_step(k) = 1/5*( (3-I)*b^k + (2+I)*(-I)^k ) - b^(k+1);
check_exact(vector(10,k,my(k=k-1); Gray_step(k)), \
            vector(10,k,my(k=k-1); Gray_end_bk(k) - b^(k+1)));

Gray_step(k) = 1/5*( (-2-6*I)*b^k + (2+I)*(-I)^k );
check_exact(vector(10,k,my(k=k-1); Gray_step(k)), \
            vector(10,k,my(k=k-1); Gray_end_bk(k) - b^(k+1)));

max_real_imag(z) =max(abs(real(z)),abs(imag(z)));
sum_real_imag(z) = abs(real(z)) + abs(imag(z));

Gray_step_max_real_imag(k) =
{
  1/5* max(abs( vector_modulo( [-2, 4, 6, 8,  2, -4, -6, -8],k)*2^floor(k/2)
               + vector_modulo([2, 1, -2, -1],k)
              )
           ,
           abs(
               vector_modulo(  [-6, -8, -2, 4,  6, 8, 2, -4],k)*2^floor(k/2)
               + vector_modulo([1, -2, -1, 2],k) 
              )
          );
}
check_exact(vector(10,k,my(k=k-1); Gray_step_max_real_imag(k)), \
            vector(10,k,my(k=k-1); max_real_imag(Gray_end_bk(k) - b^(k+1))));

Gray_step_max_real_imag(k) =
{
  1/5* max(abs( vector_modulo( [ 2, 4,  6,  8,   2,  4,  6,  8 ],k)*2^floor(k/2)
               + vector_modulo([-2, 1, -2, -1,   2, -1,  2,  1 ],k)
              )
           ,
           abs(
               vector_modulo(  [ 6, 8,  2,  4,   6,  8,  2,  4 ],k)*2^floor(k/2)
               + vector_modulo([-1, 2,  1,  2,   1, -2, -1, -2 ],k) 
              )
          );
}
check_exact(vector(10,k,my(k=k-1); Gray_step_max_real_imag(k)), \
            vector(10,k,my(k=k-1); max_real_imag(Gray_end_bk(k) - b^(k+1))));

Gray_step_max_real_imag(k) =
{
  1/5* abs( vector_modulo( [ 6, 8,  6,  8,   6,  8,  6,  8 ],k)*2^floor(k/2)
           + vector_modulo([-1, 2, -2, -1,   1, -2,  2,  1 ],k)
          );
}
check_exact(vector(10,k,my(k=k-1); Gray_step_max_real_imag(k)), \
            vector(10,k,my(k=k-1); max_real_imag(Gray_end_bk(k) - b^(k+1))));

Gray_step_max_real_imag(k) =
{
  1/5* ( vector_modulo( [ 6, 8,  6,  8,   6,  8,  6,  8 ],k)*2^floor(k/2)
        + vector_modulo([-1, 2, -2, -1,   1, -2,  2,  1 ],k)
       );
}
check_exact(vector(10,k,my(k=k-1); Gray_step_max_real_imag(k)), \
            vector(10,k,my(k=k-1); max_real_imag(Gray_end_bk(k) - b^(k+1))));


\\ print(gf_terms( (1 - 2*x + 2*x^2 + x^3 - x^4*(1 -2*x + 2*x^2 + x^3))/(1-x^8),  64 ));

g_Gray_step_max_real_imag(x) = 1/5*( (6 + 8*x)/(1-2*x^2)  - (1 - 2*x + 2*x^2 + x^3 - x^4*(1 -2*x + 2*x^2 + x^3))/(1-x^8));
check_exact(5*(gf_terms(g_Gray_step_max_real_imag(x),20) - \
               vector(20,k,my(k=k-1); max_real_imag(Gray_end_bk(k) - b^(k+1)))), \
            vector(20,k,my(k=k-1); 0));

g_Gray_step_max_real_imag(x) = 1/5*( (6 + 8*x)/(1-2*x^2)  + (-1 + 2*x - 2*x^2 -x^3)/(1 + x^4));
check_exact(5*(gf_terms(g_Gray_step_max_real_imag(x),20) - \
               vector(20,k,my(k=k-1); max_real_imag(Gray_end_bk(k) - b^(k+1)))), \
            vector(20,k,my(k=k-1); 0));

pen_up(k) =
{
  if(k==0, 0,  2*pen_up(k-1) + max_real_imag(Gray_step(k-1)));
}
print("pen_up move max ",vector(16,k,my(k=k-1); pen_up(k)))

\\ -> 2.1*2^k of hypotenuse distance
\\ -> 1.941176*2^k of longer re,im   16/17
my(k=500); pen_up(k)/2^k*1.0 
\\ my(k=20000); pen_up(k)/2^k*1.0 
print("33/17 = ",33/17.0);

pen_up_ref(k) = if(k==0, 0,  2*pen_up_ref(k-1) + max_real_imag(Gray_step(k-1)));

g_pen_up(x) = x/(1-2*x) * 1/5*( (6 + 8*x)/(1-2*x^2)  + (-1 + 2*x - 2*x^2 -x^3)/(1 + x^4));
check_exact(5*(gf_terms(g_pen_up(x),20) - \
               vector(20,k,my(k=k-1); pen_up_ref(k))), \
            vector(20,k,my(k=k-1); 0));

g_pen_up(x) = 1/17*( 33/(1-2*x) - 1/5*(170 + 238*x)/(1 - 2*x^2) + 1/5*(6*x^3 + 20*x^2 - 7*x + 5)/(1 + x^4) );
check_exact(5*(gf_terms(g_pen_up(x),20) - \
               vector(20,k,my(k=k-1); pen_up_ref(k))), \
            vector(20,k,my(k=k-1); 0));

pen_up(k) =
{
  (33/17*2^k
        - 1/85*vector_modulo( [ 170, 238 ],k)*2^floor(k/2)
        + 1/85*vector_modulo([5, -7, 20, 6, -5, 7, -20, -6],k));
}
check_exact(17*(vector(256,k,my(k=k-1); pen_up(k)) - \
                vector(256,k,my(k=k-1); pen_up_ref(k))), \
            vector(256,k,my(k=k-1); 0));

\\-----------------------------------------------------------------------------
Gray_step_sum_real_imag(k) =
{
  1/5* (abs( vector_modulo( [-2, 4, 6, 8,  2, -4, -6, -8],k)*2^floor(k/2)
               + vector_modulo([2, 1, -2, -1],k)
              )
           +
           abs(
               vector_modulo(  [-6, -8, -2, 4,  6, 8, 2, -4],k)*2^floor(k/2)
               + vector_modulo([1, -2, -1, 2],k) 
              )
          );
}
check_exact(vector(10,k,my(k=k-1); Gray_step_sum_real_imag(k)), \
            vector(10,k,my(k=k-1); sum_real_imag(Gray_end_bk(k) - b^(k+1))));


Gray_step_sum_real_imag(k) =
{
  1/5* (vector_modulo( [8, 12, 8, 12, 8, 12, 8, 12],k)*2^floor(k/2)
        + vector_modulo([-3, 3, -1, 1, 3, -3, 1, -1],k) 
       );
}
check_exact(vector(10,k,my(k=k-1); Gray_step_sum_real_imag(k)), \
            vector(10,k,my(k=k-1); sum_real_imag(Gray_end_bk(k) - b^(k+1))));

pen_up(k) =
{
  if(k==0, 0,  2*pen_up(k-1) + sum_real_imag(Gray_step(k-1)));
}
vector(10,k,my(k=k-1); pen_up(k)/2^k)

\\ -> 2.647058*2^k of sum re,im   45/17
my(k=500); pen_up(k)/2^k*1.0 
\\ my(k=20000); pen_up(k)/2^k*1.0 

\\-----------------------------------------------------------------------------
\\ pen_up() quadratic

sqrt2 = quadgen(4*2);

pen_up(k) =
{
  33/17*2^k
  + vector_modulo([-2, -7/5*sqrt2],k)*sqrt2^k
  + vector_modulo([1/17, -7/85, 4/17, 6/85, -1/17, 7/85, -4/17, -6/85],k)
  ;
}
check_exact(17*(vector(256,k,my(k=k-1); pen_up(k)) - \
                vector(256,k,my(k=k-1); pen_up_ref(k))), \
            vector(256,k,my(k=k-1); 0));

factor_vector(n) = concat(apply((r)->vector(r[2],i,r[1]), Vec(Col(factor(n)))));

{
  my(a = (k) -> 33/17);
  my(b = (k) -> vector_modulo([-2, -7/5*sqrt2],k));
  my(c = (k) -> vector_modulo([1/17, -7/85, 4/17, 6/85, -1/17, 7/85, -4/17, -6/85],k));
  my(d = (k) -> b(k)^2 - 4*a(k)*c(k));

  for(k=0,7,
     my(num = numerator(d(k)));
     my(den = denominator(d(k)));
     num = real(num);
     print(d(k),"       ",issquare(num),"  ",issquare(den),"      ", factor_vector(num)));
}
