\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

\\-----------------------------------------------------------------------------

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------

\\ p^2-q^2, 2pq with p,q not both odd means 2pq multiple of 4

\\ 1/9*(58*2^k
\\      - vector_modulo([56, 64, 28, 56],k) * 2^floor(k/2)
\\      + vector_modulo([16, 20, 4, 8],k)
\\ )

THD_quarter(k) =
{
  1/9*(29/2*2^k
       - vector_modulo([14, 16, 7, 14],k) * 2^floor(k/2)
       + vector_modulo([4, 5, 1, 2],k));
}
check_exact(vector(100,k,k--; THD_quarter(k)), \
            vector(100,k,k--; THD(k)/4))
check_exact(binary(29), [1, 1, 1, 0, 1]);

\\ never a square
\\ for(k=0,100000,if(issquare(THD(k)),print("k="k" THD="THD(k))));

T_P3_sub_P3o_real_abs(k) =
{
    1/3 * vector_modulo([3, 4, 7, 10],k) * 2^floor(k/2)
  + 1/3 * vector_modulo([0, 2, -2, -2],k);
}
check_exact(vector(100,k,k--; T_P3_sub_P3o_real_abs(k)), \
            vector(100,k,k--; abs(real(T_P3(k) - T_P3o(k)))))

T_P3_sub_P3o_imag_abs(k) =
{
    1/3 * vector_modulo([7, 10, 3, 4],k) * 2^floor(k/2)
  - 1/3 * vector_modulo([4, 4, 0, 2],k);
}
check_exact(vector(100,k,k--; T_P3_sub_P3o_imag_abs(k)), \
            vector(100,k,k--; abs(imag(T_P3(k) - T_P3o(k)))))

check_exact(vector(100,k,k--; T_P3_sub_P3o_real_abs(k)^2 + T_P3_sub_P3o_imag_abs(k)^2), \
            vector(100,k,k--; THD(k)))

THD_s_ref(k) =
{
  if(k%4==0||k%4==1, T_P3_sub_P3o_real_abs(k)/2,
                     T_P3_sub_P3o_imag_abs(k)/2);
}
THD_t_ref(k) =
{
  if(k%4==0||k%4==1, T_P3_sub_P3o_imag_abs(k)/2,
                     T_P3_sub_P3o_real_abs(k)/2);
}

THD_s(k) =
{
    vector_modulo([1/2, 2/3],k) * 2^floor(k/2)
  + vector_modulo([0, 1/3, 0, -1/3],k);
}
check_exact(vector(100,k,k--; THD_s(k)), \
            vector(100,k,k--; THD_s_ref(k)))

\\ THD_s(k) = 2^floor(k/2) for k even
check_exact(vector(100,k,k--;k*=2; THD_s(k)), \
            vector(100,k,k--;k*=2; 1/2*2^floor(k/2)), \
            "THD_s(k) power of 2")
\\ THD_s(k) even for k odd >=3
check_exact(vector(100,k,k+=2; THD_s(k) % 2), \
            vector(100,k,k+=2; k%2), \
            "THD_s(k) even/odd")

THD_t(k) =
{
          vector_modulo([7/6, 5/3],k) * 2^floor(k/2)
  - 1/3 * vector_modulo([2, 2, 1, 1],k);
}
check_exact(vector(100,k,k--; THD_t(k)), \
            vector(100,k,k--; THD_t_ref(k)), \
            "THD_t()")

\\ THD_t(k) even for k even >=2
check_exact(vector(100,k,k++;k*=2; THD_t(k) % 2), \
            vector(100,k,k++;k*=2; if(k%4==2,1,0)), \
            "THD_t(k) even/odd")


\\ no common factor
{
  for(k=0,20,
     my(k=2*k+1);
     my(g = gcd(T_P3_sub_P3o_real_abs(k),T_P3_sub_P3o_imag_abs(k)));
     if(g>1, print("k="k" gcd="g)));
}

multiplicative_order(a,m) =
{
  a %= m;
  if(a==0,error());
  my(p=a,n=1);
  while (p != 1, n++; p = (a*p)%m);
  n;
}
check_exact(multiplicative_order(1,3), 1);
check_exact(multiplicative_order(2,3), 2);
check_exact(multiplicative_order(1,5), 1);
check_exact(multiplicative_order(2,5), 4);
check_exact(multiplicative_order(3,5), 4);
check_exact(multiplicative_order(4,5), 2);

check_exact(multiplicative_order(2,3), 2);
\\ check_exact(multiplicative_order(2,4), 2);
check_exact(multiplicative_order(2,5), 4);

\\ 3,4 somewhere in A,B, ie A*B == 0 mod 12
{
  for(k=1,200,
     \\if(k%2==0,next());
     my(x = THD_s(k),
        y = THD_t(k),
        all = 1,
        missing = List([]));
     \\ if(x%2==0 && y%2==0 && z%4==0,
     \\    x/=2; y/=2; z/=4);
     for(m=3,4,
        my(xm = x%m,
           ym = y%m,
           got = (xm==0 || ym==0));
        if(!got,
           all=0;
           listput(missing,m)));
     missing = Vec(missing);
     if(all,
        print("k="k" have both 3,4");
        print("  x = "x"  ",x%3," ",x%4);
        print("  y = "y"  ",y%3," ",y%4);
        \\ break();
        ,
        print("k="k" missing A,B  "missing);
       );
     );
}

\\ 3,4,5 somewhere in A,B,C
\\ {
\\   for(k=0,200,
\\      if(k%2==0,next());
\\      my(x = T_P3_sub_P3o_real_abs(k),
\\         y = T_P3_sub_P3o_imag_abs(k),
\\         z = THD(k),
\\         all = 1,
\\         missing = List([]));
\\      \\ if(x%2==0 && y%2==0 && z%4==0,
\\      \\    x/=2; y/=2; z/=4);
\\      for(m=3,5,
\\         my(xm = x%m,
\\            ym = y%m,
\\            zm = z%(m^2),
\\            got = (xm==0 || ym==0 || zm==0));
\\         if(!got,
\\            all=0;
\\            listput(missing,m)));
\\      missing = Vec(missing);
\\      if(all,
\\         print("k="k" have all 3,4,5");
\\         print("  x = "x"  ",x%3," ",x%4," ",x%5);
\\         print("  y = "y"  ",y%3," ",y%4," ",y%5);
\\         print("  z = "z"  ",z%3," ",z%4," ",z%5);
\\         break();
\\         ,
\\         print("k="k" missing "missing);
\\        );
\\      );
\\ }


factor_vector(n) =
{
  my(v = apply((r)->vector(r[2],i,r[1]), Vec(Col(factor(n)))));
  if(#v, v = concat(v));
  v;
}
{
  for(k=1,52,
     printf("%2d %60s\n", k, factor_vector(THD_t(k))));
}
\\ k == 3 mod 4   THD(k) has 2*2*2
\\ k == 0 mod 4   Re is 2^n
\\ k == 2 mod 4   Im is 2^n

\\ vector(100,k,hammingweight(THD(k)))
\\ vector(100,k,hammingweight(THD_s(k)))
\\ vector(100,k,hammingweight(THD_t(k)))

print("mod 4");
vector(50,k,k=2*k+1; THD(k) % 4)
vector(50,k,k=2*k+1; THD_s(k) % 4)
vector(50,k,k=2*k+1; THD_t(k) % 4)


\\ % for(k=1,25,printf("%2d %60s\n", k, binary(T_P3_sub_P3o_imag_abs(k))))
\\ % strip_trailing_zeros(n) = while(n>0&&n%2==0,n/=2); n;
\\ % vector(20,k,strip_trailing_zeros(THD(k))%4)
\\ % vector(50,k,gcd(T_P3_sub_P3o_real_abs(k),T_P3_sub_P3o_imag_abs(k)))
\\ % 821%4
\\ % 29501%4
\\ % 4680857%4
\\ % mod3 is 12-period
\\ % mod4 is 4-period
\\ % mod5 is 8-period
\\ % mod25 is 40-period


\\-----------------------------------------------------------------------------

wh_direction(k) = I^(floor(k/2)+1);
T_P3_sub_P3o_width(k) =
{
  abs(real( (T_P3(k) - T_P3o(k))/wh_direction(k) ))
  * sqrt_as_quad(norm(wh_direction(k)))
  / 2;
}
T_P3_sub_P3o_height(k) =
{
  abs(imag( (T_P3(k) - T_P3o(k))/wh_direction(k) ))
  * sqrt_as_quad(norm(wh_direction(k)))
  / 2;
}

check_exact(vector(100,k,k--; 4*T_P3_sub_P3o_width(k)^2 + 4*T_P3_sub_P3o_height(k)^2), \
            vector(100,k,k--; THD(k)), \
            "width x height is THD")

vector(16,k,k--; T_P3_sub_P3o_width(k))
vector(16,k,k--; T_P3_sub_P3o_height(k))

{
  for(k=6,9,
     print("k=",k,
           " P2mid=",(T_P2(k)-b^(k+1)/2)/wh_direction(k),
           " P3mid=",(T_P3(k)-b^(k+1)/2)/wh_direction(k),
           " P4mid=",(T_P4(k)-b^(k+1)/2)/wh_direction(k),
           "    width ",T_P3_sub_P3o_width(k),
           "    height ",T_P3_sub_P3o_height(k)
          ));
}

check_exact(vector(10,k,k--; T_P3_sub_P3o_height(k)), \
            vector(10,k,k--; THD_s(k)), \
           "height")

check_exact(vector(10,k,k--;k*=2; T_P3_sub_P3o_width(k)), \
            vector(10,k,k--;k*=2; THD_t(k)), \
           "width")



system("sleep 1");