\\ Copyright 2015, 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------
vector_non_zeros(v) = select((x)->x!=0, v);

str_poly_ascending__join_strings(v,sep) =
{
  concat(vector(2*#v-1,i, if(i%2,v[(i+1)\2],sep)));
}

str_poly_ascending(p) =
{
  if(type(p) == "t_VEC",
     return(str_poly_ascending__join_strings(apply(str_poly_ascending,p),
                                             "\n + ")));

  my(d = denominator(p));
  if(poldegree(d) != 0,
     p = numerator(p);
     my(low = polcoeff(d, 0));
     if(low==0, error("Oops, constant term of denominator is 0"));
     d /= low;
     p /= low;
     my(fra=lcm(vector_non_zeros(apply(denominator,Vec(p)))));
     if(fra==0, error("oops fra=",fra));
     p *= fra;
     my(p_single = (poldegree(p) == 0
                    || poldegree(p) == valuation(p,variable(p))));
     my(d_single = (poldegree(d) == 0
                    || poldegree(d) == valuation(d,variable(d))));
     p_single = 0;
     d_single = 0;
     return(concat([if(fra==1, "", Str(1/fra,"*")),
                    if(p_single,"","("),
                    str_poly_ascending(p),
                    if(p_single,"",")"),
                    "/",
                    if(d_single,"","("),
                    str_poly_ascending(d),
                    if(d_single,"",")")
                   ])));
  if(poldegree(p) <= 0, return(Str(p)));
  my(str="");
  for(i=0,poldegree(p),
     my(coeff=polcoeff(p,i));
     if(coeff == 0, next());
     if(str!="",
        if(real(coeff)<0 || (real(coeff)==0 && imag(coeff)<0),
           str=concat(str," - "); coeff = -coeff,
           str=concat(str," + ")));
     str = concat([str,coeff*variable(p)^i]));
  str;
}

\\ str = str_poly_ascending( (x^2 + 1/2*x + 1)/(2*x^3 + x - 1) );
\\ print("str = "str);
\\ 
\\ str = str_poly_ascending( ((-1/4 + 1/4*I)*x^2 + (1/8 + 1/8*I))/(x^3 - 1/4*I*x + (1/8 + 1/8*I)) );
\\ print("str = "str);
\\ 
\\ str = str_poly_ascending( (x^2)/(2*x^3 + x - 1) );
\\ print("str = "str);

\\-----------------------------------------------------------------------------

blob_centroid_sum(k) = vecsum(blob_midpoints(k));
blob_centroid(k) = vector_mean(blob_midpoints(k));
check_exact(blob_centroid(4), -5/2+3/2*I)
check_exact(blob_centroid_sum(4), -10 + 6*I)

print("GSf = ",GS(100)/b^100 * 1.0);

\\-----------------------------------------------------------------------------
\\        9----8    5----4
\\        |    |    |    |
\\       10--11,7---6    3----2
\\             |              |
\\  16   13---12         0----1
\\   |    |   -2
\\  15---14

\\ middle of bridge after blob k
BridgeMiddle(k) = (BlobEnd(k) + BlobStart(k+1))/2;
print("BridgeMiddle ",vector(10,k,my(k=k+3); BridgeMiddle(k)))
check_exact(BridgeMiddle(4), -5/2)

BridgeMiddle_ref = BridgeMiddle;
check_exact(vector(100,k,my(k=k+3); BridgeMiddle(k)), \
            vector(100,k,my(k=k+3); BridgeMiddle_ref(k))) /* k>=4 */

BridgeMiddle(k) = 1/2*( 1/3*(2*b^k - I* boff(k-1))  +  1/3*(conj(b)*b^(k+1) + boff(k+1)));
check_exact(vector(100,k,my(k=k+3); BridgeMiddle(k)), \
            vector(100,k,my(k=k+3); BridgeMiddle_ref(k))) /* k>=4 */

BridgeMiddle(k) = 1/6*( 2*b^k  +  2*b^k + boff(k+1)  - I* boff(k-1));
check_exact(vector(100,k,my(k=k+3); BridgeMiddle(k)), \
            vector(100,k,my(k=k+3); BridgeMiddle_ref(k))) /* k>=4 */

\\       -1+2*I    
\\               I        2+I
\\         -1    o   1
\\  -2-I        -I
\\                  1-2*I
vector(8,k,my(k=k-1); boff(k+1)  - I* boff(k-1))
BridgeMiddle(k) = 2/3*b^k + 1/6*vector_modulo([-1, -1+2*I, I, 2+I,   1, 1-2*I, -I, -2-I], k);
check_exact(vector(100,k,my(k=k+3); BridgeMiddle(k)), \
            vector(100,k,my(k=k+3); BridgeMiddle_ref(k))) /* k>=4 */

gboff(x) = 1/(1-x^8)*Polrev(boff_vector);
check_exact(gf_terms(gboff(x),100), vector(100,k,my(k=k-1); boff(k)))

read("pol-pfrac.gp")
\\ print(str_poly_ascending(pol_partial_fractions(gboff(x))))
check_exact(  1/2*((-2-I)  -         x)/(1 + I*x^2)  \
            + 1/2*((6+3*I) + (3+6*I)*x)/(1 - I*x^2), \
            gboff(x));

gBridgeMiddle(k) =
{
  2/3*1/(1-b*x) + 1/6*1/(1-x^8)*Polrev([-1, -1+2*I, I, 2+I,   1, 1-2*I, -I, -2-I])
  ;
}
check_exact(gf_terms(gBridgeMiddle(x),100), vector(100,k,my(k=k-1); BridgeMiddle(k)))

\\ print(str_poly_ascending(pol_partial_fractions(gBridgeMiddle(x))))
check_exact(  2/3*               1/(1 - (1+I)*x) \
            + 1/6*(-1 - (1-2*I)*x)/(1 + I*x^2),  \
            gBridgeMiddle(x));

\\-----------------------------------------------------------------------------

GSN(k) = GS(k)*2^k;
gGSN(x) = gGS(2*x);
check_exact(gf_terms(gGSN(x),100), vector(100,k,my(k=k-1); GSN(k)))
check_exact((2/5-1/5*I)/(1 - 2*b*x) + (1/10+1/5*I)/(1 - conj(b)*x), gGSN(x));

GSstartN_by_points(k) = if(k<4,0, vecsum(dragon_midpoints(k)[1..BlobN(k+1)-3]));
my(k=4); check_exact(BlobN(k+1)-3, 11);
{
  check_exact(GSstartN_by_points(4), vecsum([1/2,
                                             1 + 1/2*I,
                                             1/2 + I,
                                             3/2*I,
                                             -1/2 + 2*I,
                                             -1 + 3/2*I,
                                             -3/2 + I,
                                             -2 + 3/2*I,
                                             -5/2 + 2*I,
                                             -3 + 3/2*I
                                             -5/2 + I]));
}
print("GSstartN_by_points = ",vector(8,k,my(k=k-1); GSstartN_by_points(k)))

my(k=4); check_exact(BlobN(k+1)-3+1, 12);
GSendN_by_points(k) = if(k<4,0, my(v=dragon_midpoints(k)); vecsum(v[BlobN(k+1)-3+1..#v]));
{
  check_exact(GSendN_by_points(4), vecsum([-2 + 1/2*I,
                                           -5/2,
                                           -3 - 1/2*I,
                                           -7/2 - I,
                                           -4 - 1/2*I]));
}
check_exact(vector(6,k,my(k=k+3); GSstartN_by_points(k) + GSendN_by_points(k)), \
            vector(6,k,my(k=k+3); GSN(k))) /* k>=4 */

\\ GSN = GSstartN(k) + GSendN(k)   + 3*(BlobEnd(k)+BlobStart(k+1))/2
\\ GSN(k) = GSstartN(k) + (b^k*(BlobN(k-1)-3) + (-I)*GSstartN(k-2))  + 3*BridgeMiddle(k)
\\ GSstartN(k)  =  GSN(k) - b^k*BlobN(k-1) + 3*b^k - (-I)*GSstartN(k-2) - 3*BridgeMiddle(k)

check_exact(vector(100,k,my(k=k+4); BlobN(k-1) + BlobN(k+1) - 3), \
            vector(100,k,my(k=k+4); 2^k))  /* k>=5 */

\\       |
\\       |
\\       v
\\ ----->
\\
GSendN(k) =
{
  if(k<4, error("GSstartN is for k>=4, got k=",k));
  if(k<6, return(GSN(k) - GSstartN(k)));
  \\ 4 -- 6 5 4
  b^k*BlobN(k-1)  \\ number of segments shifted
  + (-I)*(  GSstartN(k-2)
          + 3*BridgeMiddle(k-2) );
}
GSendN = memoize(GSendN);
my(k=6); check_exact(BlobN(k-1), 14)

GSstartN(k) = 
{
  if(k<4, error("GSstartN is for k>=4, got k=",k));
  if(k<6, return(GSstartN_by_points(k)));
  GSN(k) - GSendN(k);
}
GSstartN = memoize(GSstartN);
check_exact(vector(5,k,my(k=k+3); GSstartN(k) + GSendN(k)), \
            vector(5,k,my(k=k+3); GSN(k)), \
            "GSstart + GSend = GSN")
check_exact(vector(10,k,my(k=k+3); GSstartN(k)), \
            vector(10,k,my(k=k+3); GSstartN_by_points(k)))

GSstartN(k) = 
{
  if(k<4, error("GSstartN is for k>=4, got k=",k));
  if(k<6, return(GSstartN_by_points(k)));
  GSN(k)
  - b   * b^(k-1)*BlobN(k-1) 
  + 3*I * BridgeMiddle(k-2)

  + I   * GSstartN(k-2);
}
check_exact(vector(10,k,my(k=k+3); GSstartN(k)), \
            vector(10,k,my(k=k+3); GSstartN_by_points(k)))

gGSstartN(x)  =
{
  1/(1 - I*x^2) * ( (gGSN(x)
                     - Polrev([GSN(0),
                               GSN(1),
                               GSN(2),
                               GSN(3),
                               GSN(4),
                               GSN(5)]))
                   - b* x*( gBlobN(b*x)
                           - Polrev([BlobN_formula(0),
                                     b*BlobN_formula(1),
                                     b^2*BlobN_formula(2),
                                     b^3*BlobN_formula(3),
                                     b^4*BlobN_formula(4)
                                    ]) )
                   + 3*I * x^2 * (gBridgeMiddle(x)
                                  - Polrev([BridgeMiddle(0),
                                            BridgeMiddle(1),
                                            BridgeMiddle(2),
                                            BridgeMiddle(3)
                                           ]) )
                   + GSstartN(4)*x^4
                   + GSstartN(5)*x^5
                  )
;
}
check_exact(gf_terms(gGSstartN(x),15), \
            vector(15,k,my(k=k-1); GSstartN_by_points(k)))
check_exact(gf_terms(gGSstartN(x),100), \
            vector(100,k,my(k=k-1); if(k<4,0, GSstartN(k))))

\\ print(str_poly_ascending(pol_partial_fractions(gGSstartN(x))))
{
  check_exact(gGSstartN(x),
              -1/2*I - 1/2*x - (3/2 + 1/2*I)*x^2 - (2 + 3*I)*x^3
              
              + (-1)             /(1 - (1+I)*x)
              + 1/10*(-1 - 3*I)  /(1 + (1-I)*x)
              + 1/30*(-1 + 13*I) /(1 - (1-I)*x)
              + 1/4*(1 + (1-2*I)*x)            /(1 + I*x^2)
              + 1/84*((55+50*I) - (39-70*I)*x) /(1 - I*x^2)
              + 1/35*(8 - 8*I)   /(1 - 2*(1+I)*x)
              );
}

\\ -conj(b)   b
\\         \ /
\\          o
\\           \
\\            conj(b)
GSstartN_by_powers(k) = 
{
  if(k<4, error("GSstartN_by_powers is for k>=4, got k=",k));

  8/35*(1-I)* (2*b)^k
  -                     b^k
  + 1/30*(-1 + 13*I)  * conj(b)^k
  + 1/10*(-1 - 3*I)   * (-conj(b))^k
  
  + vector_modulo([19/21 + 25/42*I,  -3/14 + 1/3*I, -25/42 + 17/42*I, -4/3 - 5/7*I,
                   -19/21 - 25/42*I,  3/14 - 1/3*I,  25/42 - 17/42*I,  4/3 + 5/7*I], k)
  ;
}

\\ for(i=0,7, print(vector(8,k,my(k=k+63+8*i); GSstartN(k) - GSstartN_by_powers(k))))

check_exact(vector(200,k,my(k=k+3); GSstartN_by_powers(k)), \
            vector(200,k,my(k=k+3); GSstartN(k)))

\\-----------------------------------------------------------------------------

initial7 = { vecsum([1/2,
                   1 + 1/2*I,
                   1/2 + I,
                   3/2*I,
                   -1/2 + 2*I,
                   -1 + 3/2*I,
                   -3/2 + I]); }
check_exact(blob_centroid_sum(4) + initial7, GSstartN(4))

gBlobGSN(x) = {
                gGSstartN(x) - x*gGSstartN(x) - 3*x*gBridgeMiddle(x)
                + Polrev([0, 3/2, 3/2 + 3*I, 9/2*I,
                          -2 - 3*I]);
}
print("gBlobGSN          ",gf_terms(gBlobGSN(x),10))
print("blob_centroid_sum ",vector(10,k,my(k=k-1); if(k<4,0, blob_centroid_sum(k))))
print("diff              ",vector(10,k,my(k=k-1); if(k<4,0, blob_centroid_sum(k))) - gf_terms(gBlobGSN(x),10))

check_exact(gf_terms(gBlobGSN(x),15), \
            vector(15,k,my(k=k-1); if(k<4,0, blob_centroid_sum(k))))

\\ print(str_poly_ascending(pol_partial_fractions(gBlobGSN(x))))
{
  check_exact(gBlobGSN(x),
              (1/2 - 1/2*I)
              + (1 + 1/2*I)*x
              + (1/2 + 5/2*I)*x^2
              - (1/2 - 2*I)*x^3
   
              + 1/35*(8 - 4*I)  /(1 - 2*b*x)
              + 1/30*(6 + 7*I)  /(1 - conj(b)*x)
              + 1/2*(-3 + I)    /(1 - b*x)
              + 1/2*(-I)        /(1 - (-conj(b))*x)
              + 1/4*((3 + I) + (2 - 2*I)*x)/(1 + I*x^2)
              + 1/84*((-15 + 11*I) - (94 - 20*I)*x)/(1 - I*x^2)
              );
}

BlobGSN(k) = gf_term(gBlobGSN(x),k);

BlobGSN_by_powers(k) = 
{
  if(k<4, error("BlobGSN is for k>=4, got k=",k));

  1/35*(8 - 4*I)* (2*b)^k
  + 1/2*(-3 + I)      * b^k
  + 1/30*(6 + 7*I)    * conj(b)^k
  + 1/2*(-I)          * (-conj(b))^k
  
  + vector_modulo([4/7 + 8/21*I, -13/21 - 11/42*I, 5/42 - 13/14*I, -31/42 - 34/21*I,
                   -4/7 - 8/21*I, 13/21 + 11/42*I, -5/42 + 13/14*I, 31/42 + 34/21*I], k)
  ;
}
check_exact(vector(100,k,my(k=k+3); BlobGSN_by_powers(k)), \
            vector(100,k,my(k=k+3); BlobGSN(k)))

BlobGSN_by_powers(k) = 
{
  if(k<4, error("BlobGSN is for k>=4, got k=",k));

  1/35*(8 - 4*I)* (2*b)^k
  + (
     1/2*(-3 + I)      * (vector_modulo([1,1,0,-1, -1,-1,0,1],k)
                           + I*vector_modulo([0,1,1,1, 0,-1,-1,-1],k))
      + 1/30*(6 + 7*I)    * (vector_modulo([1,1,0,-1, -1,-1,0,1],k)
                             - I*vector_modulo([0,1,1,1, 0,-1,-1,-1],k))
      + 1/2*(-I)      *(vector_modulo([1,-1,0,1, -1,1,0,-1],k)
                             - I*vector_modulo([0,-1,1,-1, 0,1,-1,1],k))
     ) * 2^floor(k/2)
  
  + vector_modulo([4/7 + 8/21*I, -13/21 - 11/42*I, 5/42 - 13/14*I, -31/42 - 34/21*I,
                   -4/7 - 8/21*I, 13/21 + 11/42*I, -5/42 + 13/14*I, 31/42 + 34/21*I], k)
  ;
}
check_exact(vector(200,k,my(k=k+3); BlobGSN_by_powers(k)), \
            vector(200,k,my(k=k+3); BlobGSN(k)))

BlobGSN_by_powers(k) = 
{
  if(k<4, error("BlobGSN is for k>=4, got k=",k));

  1/35*(8 - 4*I)* (2*b)^k
  + 1/30 * vector_modulo([-39 + 7*I, -32 - 14*I, -23 - 51*I, 46 - 88*I,
                           39 - 7*I,  32 + 14*I,  23 + 51*I, -46 + 88*I],k)
    * 2^floor(k/2)
  
  + 1/42 * vector_modulo([ 24 + 16*I, -26 - 11*I,  5 - 39*I, -31 - 68*I,
                          -24 - 16*I,  26 + 11*I, -5 + 39*I,  31 + 68*I], k)
  ;
}
check_exact(vector(200,k,my(k=k+3); BlobGSN_by_powers(k)), \
            vector(200,k,my(k=k+3); BlobGSN(k)))

for(k=4,12, print(BlobGSN(k)))


quit

\\-----------------------------------------------------------------------------
BlobGSf(k) = BlobGSN(k)/BlobSegments(k)/b^k;
\\ BlobSegments(k) -> 2/5*2^k
\\ so BlobGSf -> (8/35 - 4/35*I)/(2/5)
\\               = 4/7 - 2/7*I
\\ 0.5714285714285714285714285714 - 0.2857142857142857142857142857*I


print("middle blob limit");
{
  for(k=4,12,
     print(blob_centroid(k)/b^k * 1.0,
          "    ",
          BlobGSF(k)*1.0));
}

