\\ Copyright 2015, 2016, 2017, 2018, 2019 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ This is some rough code for identifying when the cubic part of a
\\ generating function is the same as in some other generating function,
\\ possibly with constant factor or index shift (factor x^(+/-n)).
\\
\\ cubics() defines a generating function, with a name, and also searches
\\ among previous defined generating functions for ways it can be written.
\\ The first other which matches is printed.
\\
\\ The target cubics() can be put last.  If it results in an unlikely or
\\ unhelpful match of say gX, then either comment out gX or move it to
\\ immediately before the target to see if something else matches before.
\\
\\ Generally there's quite a lot of repeats up to index and constant factor.
\\


default(recover,0);
read("pol-pfrac.gp");
read("dragon-defines.gp");

h = Map();
use_derivative=0;

\\ g is a t_RFRAC generating function.
\\ Return the numerator of the dragon cubic partial fraction of it.
g_cubic_term(g) =
{
  my(v = pol_partial_fractions(g));
  for(i=1,#v,
     my(n = v[i]*(1 - x - 2*x^3));
     if(poldegree(denominator(v[i]))!=0 && poldegree(denominator(n))==0,
        return(n)));
  0;
}

cubics(g,name="noname") =
{
  \\ print("cubics "name"  g = "g);
  if(use_derivative,
     g=g'*(1 - x - 2*x^3)*x);
  print(name);
  for(e=-8,20,
     my(n = g_cubic_term(g*x^e));
     if(n==0,next());
     n = Vecrev(n);
     n = Vec(n,3);
     my(g=gcd(n));
     n = n/g;
     if(n[1]<0, n=-n; g=-g);

     my(got = "");
     if(!mapisdefined(h,n,&got),
        my(description = if(e==0, name,
                            e==1, Strprintf("%s*x",name),
                            Strprintf("%s*x^%d",name,e))); 
        mapput(h, n, description);
        \\ print("insert "n" "description);
       );
     print(n," at ",e,"  ",g,"  ",got);
  );
}

cubics(gA144181(x),"gA144181")
cubics(gRQ(x),"gRQ")
cubics(gLQ(x),"gLQ")
cubics(gBQ(x),"gBQ")
cubics(gR(x),"gR")
cubics(gL(x),"gL")
cubics(gB(x),"gB")
cubics(gTS(x),"gTS")
cubics(gDB(x),"gDB")
cubics(gS(x),"gS")
cubics(gA(x),"gA")
cubics(gJA(x),"gJA")
cubics(gdJA(x),"gdJA")
cubics(gLQ1(x),"gLQ1")
cubics(gLQ2(x),"gLQ2")
cubics(gLQ3(x),"gLQ3")
cubics(gHQA(x),"gHQA")
cubics(gHQAL(x),"gHQAL")
cubics(gHQAR(x),"gHQAR")
cubics(gHQ1(x),"gHQ")
cubics(gHQ1L(x),"gHQL")
cubics(gHQ1R(x),"gHQR")
cubics(gHQ2(x),"gHQ2")
cubics(gHQ2L(x),"gHQ2L")
cubics(gHQ2R(x),"gHQ2R")
cubics(gHQ4(x),"gHQ4")
cubics(gHQ4L(x),"gHQ4L")
cubics(gHQ4R(x),"gHQ4R")
cubics(gEightB(x),"gEightB")
gDO(x) = (-1 + 9*(x) - 4*(x)^2)/(1 - (x) - 2*(x)^3);
cubics(gDO(x),"gDO")
cubics(gP(x),"gP")
cubics(gAR(x),"gAR")
cubics(gTD(x),"gTD")
cubics((1 - x)/(1 - x - 2*x^3),"matrix power")

cubics(gBlobB(x),"gBlobB")
cubics(gBlobBQ(x),"gBlobBQ")
cubics(gBlobP(x),"gBlobP")
cubics(gBlobS(x),"gBlobS") \\ same as dJA
cubics(gAL(x),"gAL")
cubics(gBlobRQ(x),"gBlobRQ")
cubics(gBlobLQ(x),"gBlobLQ")
cubics(gBlobL(x),"gBlobL")
cubics(gBlobR(x),"gBlobR")
cubics(gBlobR(x)-gBlobL(x),"gBlobRL_diff")
cubics(gBlobRQ(x)-gBlobLQ(x),"gBlobRQLQ_diff")
cubics(gPreBlobLQ(x), "gPreBlobLQ");
cubics(gPreBlobRQ(x), "gPreBlobRQ");
cubics(gPreBlobBQ(x), "gPreBlobBQ");
cubics(gJAblobRQ(x), "gJAblobRQ(k)");
cubics(gBlobVerticals(x), "gBlobVerticals");
cubics( (-5/16 - 1/8*x + 1/8*x^2)/(1 - x - 2*x^3), "AunfOut")
cubics( (-3/16 + 1/8*x - 1/8*x^2)/16 /(1 - x - 2*x^3), "BlobAcl part")
cubics( (-5/16 - 1/8*x + 1/8*x^2)/(1 - x - 2*x^3), "BlobAcr part")
cubics( (-11/144 + 1/8*x - 1/8*x^2)/(1 - x - 2*x^3), "BlobAcl by BlobA")
cubics(gBlobAL(x) - gBlobAR(x), "BlobAL - BlobAR")
cubics( (-9/32 - 1/16*x - 3/16*x^2)/(1 - x - 2*x^3), "BlobAclCumul")
cubics( (-1/2 - 1/2*x^2)/(1 - x - 2*x^3), "Acl")
cubics(gBlobAcdiff(x), "BlobAcdiff")
cubics((3 - 6*x - 2*x^2) /(1 - x - 2*x^3), "gBlobPrecedingP delta")
cubics(gBlobPrecedingP(x) - gP(x), "BlobPrecedingP - P")
cubics(gBlobPrecedingP(x),"gBlobPrecedingP")
cubics(gBa3_sub_Ba4(x),"gBa3_sub_Ba4")
cubics(gBlobLnohang(x),"gBlobLnohang")
cubics(gBlobBnohang(x),"gBlobBnohang")
cubics(gBlobRnohang(x),"gBlobRnohang")
cubics(gBoundaryPathLen(x),"gBoundaryPathLen");
cubics(gf_shift_down( (-1/32*x - 1/16*x^2)/(1 - x - 2*x^3), 0), "gEncNine cubic");
cubics(gTP(x),"gTP");
cubics(gdP(x),"gdP");
cubics(gPB(x),"gPB")
cubics(gPE(x),"gPE")

cubics(gSixB(x), "gSixB")
cubics((-1/4 + 5/4*x - x^2)/(1 - x - 2*x^3), "gfNonBnew shortfall of R+L")
cubics((4 + 2*x + 4*x^2)/(1 - x - 2*x^3), "gTW cubic part")
cubics((-1/4 - 1/2*x - 1/2*x^2)/(1 - x - 2*x^3), "gOtotalBits")
cubics((8/3 + 4/3*x + 8/3*x^2)/(1 - x - 2*x^3), "gMinusNegB cubic")
cubics(1/119* (359 + 169*x + 348*x^2)/(1 - x - 2*x^3), "gMinusRotB cubic")
cubics((15/17 + 21/68*x + 53/68*x^2)/(1 - x - 2*x^3), "gMinusDoubleB cubic")
cubics((7/16 + 3/8*x + 1/8*x^2)/(1 - x - 2*x^3), "RQhalf to Blob End");
cubics(gFifthsDiff(3), "gFifthsDiff3");

cubics(gfR(x), "gfR")
cubics(gfR(x)-gR(x), "gfR - R")
cubics(gfL(x)-gL(x), "gfL - L")
cubics(gfB(x)-gB(x), "gfB - B")
cubics(gBlobAL(x), "BlobAL")
cubics(gSeventhsNotAll(x), "gSeventhsNotAll");
cubics(gSeventhsAll(x), "gSeventhsAll");
cubics(gfNonLnew(x), "gfNonLnew")
cubics(gfNonBnew(x), "gfNonBnew")
cubics(gfNonRnew(x), "gfNonRnew")
cubics(gfR(x) - (gR(x)-gRQ3e(x)), "gfR - (R - 3eB)")
cubics(gFifteenthsDiff(1), "gFifteenthsDiff(1)"); \\ 1,2,3,4,5,8,11,20,21
cubics(gFifteenthsDiff(2), "gFifteenthsDiff(2)");
cubics(gFifteenthsDiff(3), "gFifteenthsDiff(3)");
cubics(gFifteenthsDiff(4), "gFifteenthsDiff(4)");
\\ cubics(gFifteenthsDiff(5), "gFifteenthsDiff(5)"); \\ 2k, not cubic
cubics(gFifteenthsDiff(6), "gFifteenthsDiff(8)");
cubics(gFifteenthsDiff(7), "gFifteenthsDiff(11)");
cubics(gFifteenthsDiff(7), "gFifteenthsDiff(20)");
\\ cubics(gFifteenthsDiff(7), "gFifteenthsDiff(21)");  \\ finite, not cubic
cubics(gfL(x), "gfL")
cubics(gfB(x), "gfB")
cubics(gTAwS_over_2ksub2(x), "gTAwS_over_2ksub2")
cubics(gTAwC_over_2ksub2(x), "gTAwC_over_2ksub2")
cubics(gTAwSC_over_2ksub1(x), "gTAwSC_over_2ksub1")
cubics(gTAW_over_4k(x), "gTAW_over_4k")
cubics(gMidA_shortfall(x), "gMidA_shortfall")
cubics(gA289260(x), "A289260");
cubics((-x^3 + x^2 - x)/(-2*x^4 + 2*x^3 - x^2 + 2*x - 1), "ls2rev");
\\ cubics(gRQ(x) *x/(1-x), "RQ cumulative");
\\ cubics((3 + 2*x + 4*x^2)/(1 - x - 2*x^3), "R cumulative");
\\ cubics(gR(x)+x^2*gR(x), "R(k)+R(k-2)");
\\ cubics(gR(x)+x^2*gR(x)-x^3*gR(x), "R(k)+R(k-2)-R(k-3)");
\\ cubics(gR(x)+x*gL(x), "R(k)+L(k-1)");
cubics(gRJ(x), "gRJ");          \\ which is merely 1/2*R
cubics(gR2eA(x), "gR2eA");      \\ which is merely 2*JA
cubics(gRQ1(x), "gRQ1");
cubics(gRQ2(x), "gRQ2");
cubics(gRQ3(x), "gRQ3");

use_derivative=0
\\ cubics((3 + x + 3*x^2)/(1 - x - 2*x^3), "gOXD part")

\\ same as gS(x)
\\ print("gLQ");
\\ cubics(gLQ(x))

\\ cubics(2*x*gL(x) + gL(x),"gL pair")
\\ cubics((2*x*gS(x)-gS(x)),"gS_diff")

\\cubics((57/34 + 96/17*(x/4) + 464/17*(x/4)^2)/(1 - 4*(x/4) - 128*(x/4)^3),"g")
\\cubics((-5 - 3*x - 5*x^2)/(1 - x - 2*x^3),"g")
\\cubics( (7 + 4*x + 7*x^2)/(1 - x - 2*x^3), "g" )
