\\ Copyright 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------
\\ crossing lines around hull of middle segment

{
  segs_surround = [
                   \\ anti-clockwise from -1-I
                   [ -1-I, I ],
                   [ -1-I, 1 ],
                   [ -2*I, I ],
                   [ -2*I, 1 ],
                   [ 1-I, -I ],
                   [ 1-I, 1 ],
                   [ 2, -I ],
                   [ 2, I ],
                   [ 1+I, 1 ],
                   [ 1+I, -1 ],
                   [ -1+I,  1 ],
                   [ -1+I, -I ]
                  ];
}

{
  segs_internal = [
                   [ 0, 1 ],

                   \\ internals
                   [ 0, -1 ],
                   [ 0, I ],
                   [ 0, -I ],
                   [ 1+I, -I ],
                   [ 2, -1 ],
                   [ 1-I, I ],
                   [ 1-I, -1 ]
                  ];
}
segs_internal = Set(segs_internal);
segs = Set(concat([segs_surround,segs_internal]));


\\-----------------------------------------------------------------------------
for(i=1,#segs,for(j=i+1,#segs,if(segs[i]==segs[j],print("duplicate ",segs[j]))));
check_exact(vector_unique(segs), segs, "uniqueness");

segs_print(segs) =
{
  my(m=concat([[0,1],
               apply(e->e[1],segs),      \\ segment starts
               apply(vecsum,segs) ]));   \\ segment ends
  my(y_max=vecmax(imag(m)));
  forstep(y=y_max,vecmin(imag(m)),-1,
     if(y<y_max,
       \\ above verticals
       print1("   ");
       for(x=vecmin(real(m)), vecmax(real(m)),
          if(vector_contains(segs,[x+I*y,I]),    print1("^     "),
             vector_contains(segs,[x+I*y+I,-I]), print1("|     "),
                                                 print1("      "));
          );
       print();

       print1("   ");
       for(x=vecmin(real(m)), vecmax(real(m)),
          if(vector_contains(segs,[x+I*y,I]),    print1("|     "),
             vector_contains(segs,[x+I*y+I,-I]), print1("|     "),
                                                 print1("      "));
          );
       print();

       print1("   ");
       for(x=vecmin(real(m)), vecmax(real(m)),
          if(vector_contains(segs,[x+I*y,I]),    print1("|     "),
             vector_contains(segs,[x+I*y+I,-I]), print1("v     "),
                                                 print1("      "));
          );
       print());

     \\ row
     print1("   ");
     for(x=vecmin(real(m)), vecmax(real(m)),
        print1(if(x==0&&y==0,"O","*"));
        if(vector_contains(segs,[x+I*y,1]),    print1(if(x==0&&y==0,
                                                         "====>",
                                                         "---->")),
           vector_contains(segs,[x+1+I*y,-1]), print1("<----"),
                                               print1("     "));
     );
     print();
  );
  print();
}

print("enclose");
segs_print(segs_surround);
print("internal");
segs_print(segs_internal);
print("segs");
segs_print(segs);

segs_expand0(v) = \
  concat(apply(e->my(z=e[1],d=e[2]); [ [b*z,d], [b*(z+d),d*-I] ], v));

check_exact(segs_expand0([[0,1]]), [ [0,1], [1+I,-I] ]);
check_exact(segs_expand0([[0,I]]), [ [0,I], [-1+I,1] ]);
check_exact(segs_expand0([[0,-1]]), [ [0,-1], [-1-I,I] ]);
check_exact(segs_expand0([[-1-I,1]]), [ [-2*I,1], [1-I,-I] ]);

check_exact(segs_expand0([[-1+I,-I]]), [ [-2,-I], [-1-I,-1] ]);
check_exact(segs_expand0([[-1+I,1]]), [ [-2,1], [-1+I,-I] ]);

print("number of segs ",#segs);

rotate1(v) = apply(e->my(z=e[1],d=e[2]); z-=1+I; [z*I,d*I], v);
segs_expand1(v) = rotate1(segs_expand0(v));
check_exact(segs_expand1([[0,1]]), [ [1-I,I], [0,1] ]);
check_exact(segs_expand1([[-1-I,I]]), [ [3-I,-1], [2-2*I,I] ]);

check_exact(setminus(Set(segs),Set(segs_expand0(segs))), [], "expand0");
check_exact(setminus(Set(segs),Set(segs_expand1(segs))), [], "expand1");

\\ state 0 right side enc, state 1 left side enc
\\  0 ------> 1         1 ------> 0
\\   0\     /1           1\     /0
\\ 0a  \   /  1b       1a  \   /  0b
\\     1v v0               0v v1
segs_set = Set(segs);
transition(state,bit) =
{
  if(bit==state[1],
     concat([bit],
            setintersect(Set(segs_expand0(state[^1])), segs_set)),
     concat([bit],
            setintersect(Set(segs_expand1(state[^1])), segs_set)));
}

segs_surround_set = Set(segs_surround);
set_is_subset(subset,superset) = setminus(subset,superset) == [];
state_is_enclosed(state) = set_is_subset(segs_surround_set, state[^1]);

states=List([ Set([0, [0,1]]) ]);
state_to_index(state) =
{
  for(i=1,#states,if(state==states[i],return(i)));
  error("oops, state not found: ",state);
}
state_print(state) =
{
  print("state=",state);
  print("side ",state[1], "  index=",state_to_index(state),
       " enc ",state_is_enclosed(state));
  segs_print(state[^1]);
  print("----------------");
}

{
  my(from=1);
  while(from<=#states,
        for(bit=0,1,
           my(new_state=transition(states[from],bit));
           if(!vector_contains(states,new_state),
              listput(states,new_state);
             \\ print("add ",#states);
             ));
       from++);
}
print("states ",#states);

\\ state_print(states[1]);
\\ state_print(states[2]);
\\ state_print(states[3]);
\\ state_print(states[4]);
\\ state_print(states[5]);
\\ state_print(states[6]);
\\ state_print(states[7]);
\\ state_print(transition(states[4],0));
\\ state_print(transition(states[4],1));

check_exact(transition(states[1],0), [0, [0,1], [1+I,-I]], "initial transition 0");
check_exact(transition(states[1],1), [1, [0,1], [1-I,I]],  "initial transition 1");

\\ check_exact(transition([0, [-1+I,-I]],0), [0, [-2,-I], [-1-I,-1]], "transition 0");
\\ check_exact(vector_contains(segs,[-2,-I]),1);
\\ check_exact(vector_contains(segs,[-1-I,-1]),1);

print("---------------------------------------------------------------------");
print("enclosed states");
{
  for(i=1,#states,
     my(state=states[i]);
     if(state_is_enclosed(state),
        print("enc("state[1]") ",i);
        state_print(state);
       ));
}

print("---------------------------------------------------------------------");
print("enclosures remain enclosed");
{
  my(bad=0);
  for(i=1,#states,
     my(state=states[i]);
     if(!state_is_enclosed(state),next());
     for(bit=0,1,
        my(new_state=transition(states[i],bit));
        if(state_is_enclosed(new_state),next());
        print("from");
        state_print(state);
        print("to bit="bit);
        state_print(new_state);
        print("did not remain enc\n\n\n\n");
        bad++));
  if(bad,print(bad," bad");quit(1));
}

print("---------------------------------------------------------------------");
print("table");
{
  for(i=1,#states,
     print("  "i"=>{0=>" state_to_index(transition(states[i],0)),
           ",1=>", state_to_index(transition(states[i],1)), "},"));
}

print("---------------------------------------------------------------------");
print("eventually enclosing");
eventually_depth=vector(#states,i,-1);
{
  for(i=1,#states,
     my(state=states[i]);
     if(state_is_enclosed(state),
        eventually_depth[state_to_index(state)]=0));
}

eventually_states=Set(select(state->state_is_enclosed(state),states));
state_is_enclosed_eventually(state) = setsearch(eventually_states,state) > 0;

{
  my(more=1,iterations=0);
  while(more,
        iterations++;
        more=0;
        for(i=1,#states,
           my(state=states[i]);
           if(state_is_enclosed_eventually(state),next());

           my(new_state0=transition(state,0));
           my(new_state1=transition(state,1));
           if(state_is_enclosed_eventually(new_state0)
              && state_is_enclosed_eventually(new_state1),
             eventually_states=setunion(eventually_states,[state]);

              eventually_depth[state_to_index(state)]
              = 1 + max(eventually_depth[state_to_index(new_state0)],
                        eventually_depth[state_to_index(new_state0)]);
             more=1)));
  print("iterations ",iterations);
}

\\print(eventually_states);
print(apply(state_to_index,eventually_states));
print("count ",#eventually_states);
print("leaving ",#states-#eventually_states);

{
  for(i=1,#states,
     my(state=states[i]);
     if(state_is_enclosed_eventually(state),
        print("  "i"  depth ",
              eventually_depth[state_to_index(state)])));
}

\\apply(state_print,eventually_states);


\\ print("---------------------------------------------------------------------");
\\ print("internals which not eventually enclosing");
\\ \\ various of these
\\ 
\\ state_is_full_internal(state) = setminus(segs_internal,state[^1])==[];
\\ check_exact(state_is_full_internal(states[90]),1, \
\\             "internals of surrounded enc");
\\ {
\\   for(i=1,#states,
\\      my(state=states[i]);
\\      if(state_is_full_internal(state)
\\         && !state_is_enclosed_eventually(state),
\\         state_print(state);
\\        ));
\\ }

print("---------------------------------------------------------------------");

transitions(state,v) = for(i=1,#v,state=transition(state,v[i])); state;

fNonBpred_final_state(f) =
{
  my(v=frac_to_fracper(f,2));
  my(state=transitions(states[1],v[^-1])); \\ fixed part
  my(seen=[]);
  v=v[#v];  \\ periodic part
  while(1,
        seen=setunion(seen,[state]);
        state=transitions(state,v);
        if(setsearch(seen,state),break()));
  state;
}
fNonBpred_by_states(f) = state_is_enclosed_eventually(fNonBpred_final_state(f));
fBpred_by_states(f) = !fNonBpred_by_states(f);

vector_bit_flip(v) = apply(n->1-n,v);
fracper_bit_flip(v) = \
  concat(vector_bit_flip(v[^-1]), [vector_bit_flip(v[#v])]);
check_exact(fracper_bit_flip([1,1,0, [0,1,1,0]]), [0,0,1, [1,0,0,1]]);

\\-----------------------------------------------------------------------------


Rpred_transitions_of_fracper(state,v) =
{
  my(state=Rpred_transitions(state,v[^-1]));  \\ fixed part
  my(seen=[]);
  v=v[#v];  \\ periodic part
  while(1,
        seen=setunion(seen,[state]);
        state=Rpred_transitions(state,v);
        if(setsearch(seen,state),break()));
  state;
}
fRpred_by_RL_of_fracper(v) = Rpred_transitions_of_fracper(state,v)!=Rpred_non;
fRpred_by_RL(f) =
{
  my(fv=frac_to_fracper(f,2));
  my(state=Rpred_transitions_of_fracper(Rpred_1e,fv));
  state!=Rpred_non;
}
fLpred_by_RL(f) =
{
  my(fv=frac_to_fracper(f,2));
  fv=fracper_bit_flip(fv);
  my(state=Rpred_transitions_of_fracper(Rpred_1o,fv));
  state!=Rpred_non;
}
fNonBpred_by_RL(f) = !fRpred_by_RL(f) && !fLpred_by_RL(f);
fBpred_by_RL(f) = fRpred_by_RL(f) || fLpred_by_RL(f);

Rpred_state_name = ["1e", "1o", "2eA", "3oA", "3oB", "2eB_3oC", "3eA", "3eB", "3eC", "non"];
show_Rpred_transitions_of_fracper(state,fv) =
{
  print(fv" at "state" "Rpred_state_name[state]);
  for(i=1,#fv-1,
     my(bit=fv[i]);
     state=Rpred_transition(state,bit);
     print("bit "bit" to "state" "Rpred_state_name[state]));
  my(seen=[]);
  fv=fv[#fv];  \\ periodic part
  print("periodic");
  while(1,
        seen=setunion(seen,[state]);
  for(i=1,#fv,
     my(bit=fv[i]);
     state=Rpred_transition(state,bit);
     print("bit "bit" to "state" "Rpred_state_name[state]));
        if(setsearch(seen,state),break()));
  state;
}


\\-----------------------------------------------------------------------------
\\ NonBpred by cases

\\ n = num mod 30
\\ n+1 after
\\ n-11 before is n+19 pred at 2*n+30
\\
\\ 15*(2*n+1)-7 == 30*n+8
\\ 15*(2*n+1)+7 == 30*n+22

fBpred_by_cases(f) =
{
  my(verbose=1);
  if(verbose,print("fBpred_by_cases() "f));

  my(den=denominator(f));
  if(den%15==0,
     my(pow=den/15);
     if(is_power_of_2(pow),
        my(k=logint(pow,2));
        if(den!=15*2^k,error());

        my(num=numerator(f));
        my(n=num\30,r=num%30);
        if(r>15, r-=30;n++);
        if(r<=-15 || r>15,error("oops"));
        my(fback=(2*n + r/15)/2^k);
        if(f!=fback,error("oops"));
        if(verbose,print("  n="n" k="k" r="r));

        \\ 2n+1/15 or +11/15 on right boundary of subcurve
        \\    <---11------ 2n --1------> 2n+1
        \\          \       | /
        \\           ------ 3
        \\                  |
        \\                  v
        if(abs(r)==1 || abs(r)==11,
           if(verbose,print("  case +/-1 or +/-11, Spred(2n)="Spred(k,2*n)));
           return(Spred(k,2*n)));

        \\ \\ 2n-1/15 or -1/15 on right boundary of subcurve
        \\ \\    <---11----- 2n+2 --1------> 2n+1
        \\ \\          \       |  /
        \\ \\           ------ 3
        \\ \\                  |
        \\ \\                  v
        \\ if(r==19 || r==29, return(Spred(k,2*n+2)));

        \\ 2n+7/15 on left boundary of half subcurve,
        \\ enclosed by bridge 9 but on boundary if no 13 with is S left
        \\             /\
        \\     2n ----7--9--> 2n+1
        \\                \  ^
        \\                 \ |
        \\                  13
        \\                   |     big
        \\                    2n+2
        if(r==7,
           if(verbose,print("  case 2n+7/15, Spred(2n+1)="Spred(k,2*n+1),
                            " turn(2n+1)="turn(2*n+1)));
           return(Spred(k,2*n+1) && turn(2*n+1)==1));
        
        \\ if(r<0, r+=30;n--);
        \\ if(!(2*n+1>=1 && (2*n+1)%2==1), error("oops, bad 2n+1 =", 2*n+1));
        r-=30;
        if(r<=-15, r+=30;n--);
        fback=(2*n+2 + r/15)/2^k;
        if(f!=fback,error("oops f="f" fback="fback));
        if(r<=-15 || r>15,error("oops r="r));

        \\ 2n+23 = (2n+2)-7 enclosed by bridge 9 but on boundary if no 13
        \\ enclosed by bridge 9 but on boundary if no 13
        \\             /\                         /\
        \\   2n+2 ----7--9--> 2n+1        2n ----7--9--> 2n-1
        \\                \  ^                       \  ^
        \\                 \ |                        \ |
        \\                  13                         13
        \\                   |                          |
        \\                    2n                         2n-2
        if(r==-7,
           if(verbose,print("  case 2n+2-7/15, Spred(2n+1)="Spred(k,2*n+1),
                            " turn(2n+1)="turn(2*n+1)));
           return(Spred(k,2*n+1) && turn(2*n+1)==-1));

        \\ if(r==23, return(Spred(k,2*n+1) && turn(2*n+1)==-1));

        \\ if(r>15, r-=30;n++);
        \\ if(!(2*n+1>=1 && (2*n+1)%2==1), error("oops, bad 2n+1 =", 2*n+1));
        \\ r-=15;
        \\ if(r<=-15, r+=30;n--);
        \\ fback=(2*n+1 + r/15)/2^k;
        \\ if(f!=fback,error("oops f="f" fback="fback));
        \\ if(r<=-15 || r>15,error("oops r="r));

        \\ if(r==-8, return(Spred(k,2*n+1) && turn(2*n+1)==1));
        \\ if(r==8, return(Spred(k,2*n+1) && turn(2*n+1)==-1));

        \\ if(r==-7, return(Spred(k,2*n-1) && turn(2*n-1)==-1));
       ));

  fBpred_by_RL(f);
}

\\ check_exact(fRpred_by_cases(7/15),1, "Rpred_by_cases 7/15");
\\ check_exact(fLpred_by_cases(7/15),0, "Lpred_by_cases 7/15");

\\-----------------------------------------------------------------------------
print("sevenths");
frac_to_fracper(8/15,2)
for(i=1,6,print(i,"  ",fBpred_by_states(i/7)," ",fBpred_by_cases(i/7)));

print("fifteenths");
for(i=1,14,print(i,"  ",fBpred_by_states(i/15)," ",fBpred_by_cases(i/15)));

print("KM ",fBpred_by_states(KM_sum(8)));
frac_to_fracper(KM_sum(6),2)


\\-----------------------------------------------------------------------------
print("15ths fractions");

\\ 11011 0010 0010
f=(from_binary(101) + 7/15)/8
f=7/15
binary_padded(floor(f*2^16), 16)
print("fBpred_by_states ",fBpred_by_states(f));
print("fBpred_by_cases ", fBpred_by_cases(f));
quit
show_Rpred_transitions_of_fracper(Rpred_1e,frac_to_fracper(f,2));
show_Rpred_transitions_of_fracper(Rpred_1o,fracper_bit_flip(frac_to_fracper(f,2)));
{
  for(k=0,9,
     my(n=floor(f*2^k));
     my(state=Rpred_transitions(Rpred_1e,binary(n)));
     print("k="k" n="n" state "Rpred_state_name[state]));
}

{
  my(m=15, bad=0);
  for(k=0,10,
     my(den=m*2^k);
     for(num=1,den-1,
        if(gcd(num,den)!=1,next());
        my(f=num/den);
        my(o=num % (2*m));
        if(o>m,o-=2*m);
        my(s=fBpred_by_cases(f));
        my(t=fBpred_by_states(f));
        if(s!=t,
           my(z=dragon_fpoint(f));
           \\ if(abs(o)==1,next());
           \\ if(abs(o)==7,next());
           \\ if(abs(o)==11,next());
           print("m="m" "num"/"den" o="o
                 "  cases="s"  states="t"  at "z
                 "  R="fRpred_by_RL(f),
                 " L="fLpred_by_RL(f));
           bad++;
           if(bad>5,quit))));
}
quit

f=147/224; print("f="f);
frac_to_fracper(f,2)
show_Rpred_transitions_of_fracper(Rpred_1e,fracper_bit_flip(frac_to_fracper(f,2)))
{
  for(k=0,10,
     my(n=floor(f*2^k));
     print("Lpred "n" of "2^k" = "Lpred(k,n));
  );
}

state_print(fBpred_final_state(f))

\\ k=10;
\\ floor(3/7*2^k)
\\ ceil(3/7*2^k)
\\ dragon_point(floor(3/7*2^k))
\\ dragon_point(ceil(3/7*2^k))

v=frac_to_fracper(f,2)
state=transitions(states[1],v[^-1]); \\ fixed part
seen=[];
v=v[#v]  \\ periodic part
{
while(1,
      state_print(state);
      seen=setunion(seen,[state]);
      state=transitions(state,v);
      if(setsearch(seen,state),break()));
}
for(i=1,#v,state_print(state);state=transition(state,v[i]));

state_is_enclosed_eventually(state)


\\-----------------------------------------------------------------------------
\\ within k bits


\\ print("runs");
\\ {
\\   my(state=states[1]);
\\   for(i=1,10,
\\      state_print(state);
\\      state=transition(state,0);
\\
\\      state_print(state);
\\      state=transition(state,0);
\\
\\      state_print(state);
\\      print("now 1");
\\      state=transition(state,1);
\\
\\      );
\\ }
