#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve;

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyDragon 'BlobStart_xy','BlobEnd_xy';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 16;
if (@ARGV) { $max_level = shift @ARGV; }

$| = 1;
print "% generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

my $var = $FindBin::Script;
$var =~ s/\.pl$//;
$var =~ tr/-/_/;

#    ----   ----
#   |     |     |
#    --- x,y ---
#   |     |     |
#    ----   ----

sub xy_is_boundary_point {
  my ($x,$y) = @_;
  # upwards
  foreach my $dx (-1, 0, 1) {
    foreach my $dy (-1, 0) {
      if (! defined $path->xyxy_to_n_either($x+$dx,$y+$dy, $x+$dx, $y+$dy+1)) {
        return 1;
      }
    }
  }
  # rightwards
  foreach my $dx (-1, 0) {
    foreach my $dy (-1, 0, 1) {
      if (! defined $path->xyxy_to_n_either($x+$dx,$y+$dy, $x+$dx+1, $y+$dy)) {
        return 1;
      }
    }
  }
  return 0;
}

print "% GP-DEFINE  ${var}_generated_samples = [";

foreach my $k (0 .. $max_level) {
  if ($k < 4) { print "0,"; next; }

  my ($x,$y) = BlobStart_xy($k);
  my ($x_end,$y_end) = BlobEnd_xy($k);
  ### delta: "$x,$y to $x_end,$y_end"
  my $dx = $x_end - $x;
  if ($dx) { $dx /= abs($dx); }
  my $dy = $y_end - $y;
  if ($dy) { $dy /= abs($dy); }

  my $BlobDeltaPB = 0;
  do {
    if (xy_is_boundary_point($x,$y)) {
      $BlobDeltaPB++;
    }
    $x += $dx;
    $y += $dy;
  } until ($x == $x_end && $y == $y_end);
  print $BlobDeltaPB, ($k==$max_level ? '' : ',');
}

print "];\n";

print "% end generated\n";
exit 0;
