\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\ngot  %s\nwant %s",x,y)));
  return(0);
}

check_equal(got,want,name="") =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong ",name,"\n",d); quit(1),
     print("ok ",check_count," ",name));
}

\\-----------------------------------------------------------------------------

cis(a) = cos(a) + I*sin(a);

Ika_ref(k,a) =
{
  my(v=dragon_midpoints(k));
  my(centroid = GS(k));
  my(rotate = 1/((b/sqrt(2))^k * cis(a)));
  v = apply(z->(z-centroid)*rotate, v);
  sum(i=1,#v, imag(v[i])^2);
}
Ika_ref = memoize(Ika_ref);

\\ imag(p/cis(a))^2
\\ = imag((x+I*y)*(cos(a) - I*sin(a)))^2

Ix_ref(k) =
{
  my(v=dragon_midpoints(k));
  my(centroid = GS(k));
  my(rotate = 1/b^k);
  v = apply(z->(z-centroid)*rotate, v);
  sum(i=1,#v, imag(v[i])^2) * 2^k;
}
Ix_ref = memoize(Ix_ref);
check_equal(vector(14,k,my(k=k-1); Ix_ref(k)), \
            vector(14,k,my(k=k-1); Ika_ref(k,0)), \
            "Ix_ref")

Iy_ref(k) =
{
  my(v=dragon_midpoints(k));
  my(centroid = GS(k));
  my(rotate = 1/b^k);
  v = apply(z->(z-centroid)*rotate, v);
  sum(i=1,#v, real(v[i])^2) * 2^k;
}
Iy_ref = memoize(Iy_ref);
check_equal(vector(14,k,my(k=k-1); Iy_ref(k)), \
            vector(14,k,my(k=k-1); Ika_ref(k,Pi/2)), \
            "Iy_ref")

Ixy_ref(k) =
{
  my(v=dragon_midpoints(k));
  my(centroid = GS(k));
  my(rotate = 1/b^k);
  v = apply(z->(z-centroid)*rotate, v);
  -2 * sum(i=1,#v, imag(v[i])*real(v[i])) * 2^k;
}
Ixy_ref = memoize(Ixy_ref);
{
  check_equal(vector(40,a,my(a=2*Pi/a);
              vector(10,k,my(k=k-1);
         Ix_ref(k)*cos(a)^2 + Ixy_ref(k)*cos(a)*sin(a) + Iy_ref(k)*sin(a)^2
              )),
    vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika_ref(k,a))),
    "Ixy_ref")
}

Iz_ref(k) =
{
  my(v=dragon_midpoints(k));
  my(centroid = GS(k));
  v = apply(z->(z-centroid), v);
  sum(i=1,#v, norm(v[i]));
}
Iz_ref = memoize(Iz_ref);
check_equal(vector(14,k,my(k=k-1); Iz_ref(k)), \
            vector(14,k,my(k=k-1); Ix_ref(k) + Iy_ref(k)))

\\-----------------------------------------------------------------------------

I_GS_diff_ref(k) = GS(k) - GS(k-1);

I_GS_diff(k) =
{
  (2/5 - 1/5*I)*b^k + (1/10 + 1/5*I)*conj(b/2)^k
  - (
     (2/5 - 1/5*I)*b^(k-1) + (1/10 + 1/5*I)*conj(b/2)^(k-1)
    );
}
check_exact(vector(100,k, I_GS_diff(k)), \
            vector(100,k, I_GS_diff_ref(k))) /* k>=1 */

I_GS_diff(k) =
{
  (2/5 - 1/5*I)*b^k
  - (2/5 - 1/5*I)/b *b^k
  + (1/10 + 1/5*I)*conj(b/2)^k
  - (1/10 + 1/5*I)/conj(b/2) *conj(b/2)^k
  ;
}
check_exact(vector(100,k, I_GS_diff(k)), \
            vector(100,k, I_GS_diff_ref(k))) /* k>=1 */

I_GS_diff(k) =
{
  (3/10 + 1/10*I) * b^k   +   (1/5 - 1/10*I) * conj(b/2)^k;
}
check_exact(vector(100,k, I_GS_diff(k)), \
            vector(100,k, I_GS_diff_ref(k))) /* k>=1 */

\\-----------------------------------------------------------------------------

I_GS_norm_ref(k) = norm(GS(k) - GS(k-1));

I_GS_norm(k) =
{
  norm(
       2^floor(k/2)
       *(3/10 + 1/10*I)
       * (vector_modulo([1,1,0,-1, -1,-1,0,1],k)
          + I*vector_modulo([0,1,1,1, 0,-1,-1,-1],k))
       + 
       2^floor(k/2)
       * (1/5 - 1/10*I )*1/2^k 
       * (vector_modulo([1,1,0,-1, -1,-1,0,1],k)
          - I*vector_modulo([0,1,1,1, 0,-1,-1,-1],k))

      );
}
check_exact(vector(100,k, I_GS_norm(k)), \
            vector(100,k, I_GS_norm_ref(k))) /* k>=1 */

I_GS_norm(k) =
{
  norm(
       2^floor(k/2)
       * vector_modulo([3/10 + 1/10*I, 1/5 + 2/5*I, -1/10 + 3/10*I, -2/5 + 1/5*I, -3/10 - 1/10*I, -1/5 - 2/5*I, 1/10 - 3/10*I, 2/5 - 1/5*I],
                       k)
       + 
       1/2^ceil(k/2)
       * (vector_modulo([1/5 - 1/10*I, 1/10 - 3/10*I, -1/10 - 1/5*I, -3/10 - 1/10*I, -1/5 + 1/10*I, -1/10 + 3/10*I, 1/10 + 1/5*I, 3/10 + 1/10*I],k))
      );
}
check_exact(vector(100,k, I_GS_norm(k)), \
            vector(100,k, I_GS_norm_ref(k))) /* k>=1 */

I_GS_norm(k) =
{
  norm(
       2^floor(k/2)
       * vector_modulo([3/10, 1/5, -1/10, -2/5, -3/10, -1/5, 1/10, 2/5],
                       k)
       + 
       1/2^ceil(k/2)
       * (vector_modulo([1/5, 1/10, -1/10, -3/10, -1/5, -1/10, 1/10, 3/10],k))
 
       + I*(
            2^floor(k/2)
            * vector_modulo([1/10, 2/5, 3/10, 1/5, - 1/10, - 2/5, - 3/10, - 1/5], k)
            + 1/2^ceil(k/2)
            * (vector_modulo([-1/10, -3/10, -1/5, -1/10, 1/10, 3/10, 1/5, 1/10],k))
           )

      );
}
check_exact(vector(100,k, I_GS_norm(k)), \
            vector(100,k, I_GS_norm_ref(k))) /* k>=1 */


I_GS_norm(k) =
{
  1/10 * 2^k
  + vector_modulo([1/10, -1/10, -1/10, 1/10],k)
  + 1/20 * 1/2^k
  ;
}
check_exact(vector(100,k, I_GS_norm(k)), \
            vector(100,k, I_GS_norm_ref(k))) /* k>=1 */

check_exact(gf_terms(1/(1-x^4) * Polrev([1/10, -1/10, -1/10, 1/10]),100), \
            vector(100,k,my(k=k-1); \
                         vector_modulo([1/10, -1/10, -1/10, 1/10],k)))

gI_GS_norm(x) = 
{
  1/10 * 1/(1 - 2*x)
  + 1/(1-x^4) * Polrev([1/10, -1/10, -1/10, 1/10])
  + 1/20 * 1/(1 - 1/2*x)
  ;
}
check_exact(gf_terms(gI_GS_norm(x),100), \
            vector(100,k,my(k=k-1); I_GS_norm_ref(k)), "gI_GS_norm")

{
  check_exact((1/20)/(1 - 1/2*x)
              + (1/10)/(1 - 2*x)
              + (1/10 - 1/10*x)/(1 + x^2)
              ,
              gI_GS_norm(x))
}


\\-----------------------------------------------------------------------------
\\ cos(a+45) = cos(a)*1/sqrt(2) - sin(a)*1/sqrt(2)

{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + Pi/4),
                                1/sqrt(2)*cos(a) - 1/sqrt(2)*sin(a))),
            vector(100,k, 1))
}
{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + Pi/4)^2,
                                1/2*cos(a)^2 - cos(a)*sin(a) + 1/2*sin(a)^2)),
            vector(100,k, 1))
}
{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + Pi/4)^2,
                                1/2 - cos(a)*sin(a))),
            vector(100,k, 1))
}

{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + 3*Pi/4),
                                - 1/sqrt(2)*cos(a) - 1/sqrt(2)*sin(a))),
            vector(100,k, 1))
}
{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + 3*Pi/4)^2,
                                1/2 + cos(a)*sin(a))),
            vector(100,k, 1))
}

\\-----------------------------------------------------------------------------
\\ sin(a+45) = cos(a)*1/sqrt(2) + sin(a)*1/sqrt(2)

{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(sin(a + Pi/4),
                                1/sqrt(2)*cos(a) + 1/sqrt(2)*sin(a))),
            vector(100,k, 1))
}
{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(sin(a + Pi/4)^2,
                                1/2 + cos(a)*sin(a))),
            vector(100,k, 1))
}

{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(sin(a + 3*Pi/4),
                                1/sqrt(2)*cos(a) - 1/sqrt(2)*sin(a))),
            vector(100,k, 1))
}
{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(sin(a + 3*Pi/4)^2,
                                1/2 - cos(a)*sin(a))),
            vector(100,k, 1))
}

\\-----------------------------------------------------------------------------
\\ cos(a+45)*sin(a+45) 

{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + Pi/4)*sin(a + Pi/4),
                                1/2*cos(a)^2 - 1/2*sin(a)^2)),
            vector(100,k, 1))
}
{
check_exact(vector(100,i,my(a=2*Pi/i);
                   nearly_equal(cos(a + 3*Pi/4)*sin(a + 3*Pi/4),
                                - 1/2*cos(a)^2 + 1/2*sin(a)^2)),
            vector(100,k, 1))
}


\\-----------------------------------------------------------------------------

I_GS_rot_ref(k) = I_GS_diff(k) * exp(I*-Pi/4*k);

I_GS_rot(k) =
{
  ((3/10 + 1/10*I) * b^k   +   (1/5 - 1/10*I) * conj(b/2)^k
) * exp(I*-Pi/4*k);
}
check_equal(vector(100,k, I_GS_rot(k)), \
            vector(100,k, I_GS_rot_ref(k))) /* k>=1 */

check_equal(exp(I*-Pi/4), (1/sqrt(2) - I*1/sqrt(2)))

I_GS_rot(k) =
{
  ((3/10 + 1/10*I) * b^k   +   (1/5 - 1/10*I) * conj(b/2)^k
) * (1/sqrt(2) - I*1/sqrt(2))^k;
}
check_equal(vector(100,k, I_GS_rot(k)), \
            vector(100,k, I_GS_rot_ref(k))) /* k>=1 */

I_GS_rot(k) =
{
  (3/10 + 1/10*I) * (b*(1/sqrt(2) - I*1/sqrt(2)))^k
  +
  (1/5 - 1/10*I) * conj(b/2*(1/sqrt(2) + I*1/sqrt(2)))^k
;
}
check_equal(vector(100,k, I_GS_rot(k)), \
            vector(100,k, I_GS_rot_ref(k))) /* k>=1 */

I_GS_rot(k) =
{
  (3/10 + 1/10*I) * (sqrt(2))^k
  +
  (1/5 - 1/10*I) * (-I/sqrt(2))^k
;
}
check_equal(vector(100,k, I_GS_rot(k)), \
            vector(100,k, I_GS_rot_ref(k))) /* k>=1 */

check_equal(vector(20,k, norm(I_GS_rot(k))), \
            vector(20,k, I_GS_norm_ref(k))) /* k>=1 */

\\-----------------------------------------------------------------------------

I_GS_rot_sqrt2k_ref(k) = I_GS_rot_ref(k) * sqrt(2)^k;

I_GS_rot_sqrt2k(k) =
{
  (3/10 + 1/10*I) * (sqrt(2))^k * (sqrt(2))^k
  +
  (1/5 - 1/10*I) * (-I/sqrt(2))^k * (sqrt(2))^k
  ;
}
check_equal(vector(50,k, I_GS_rot_sqrt2k(k)), \
            vector(50,k, I_GS_rot_sqrt2k_ref(k))) /* k>=1 */

I_GS_rot_sqrt2k(k) =
{
  (3/10 + 1/10*I) * 2^k  +  (1/5 - 1/10*I) * (-I)^k
  ;
}
check_equal(vector(50,k, I_GS_rot_sqrt2k(k)), \
            vector(50,k, I_GS_rot_sqrt2k_ref(k))) /* k>=1 */

\\-----------------------------------------------------------------------------
\\ Ika()

\\                   *
\\                   |
\\                   |
\\                   +
\\                   |
\\               +   |
\\                   |
\\    *-------+------*

\\ sin(a - arg(I_GS_diff))^2
\\ = (sin(a)*cos(diff) - cos(a)*sin(diff))^2
\\ r^2*( sin(a)^2*cos(diff)^2 - 2*cos(a)*sin(diff)*sin(a)*cos(diff) + cos(a)^2*sin(diff)^2 )
\\ = sin(a)^2*real(diff)^2 - 2*cos(a)*sin(a)*imag(diff)*real(diff) + cos(a)^2*imag(diff)^2
\\
\\ r^2*cos(arg(diff))^2 = real(diff)^2
\\
\\ (     imag(I_GS_diff(k))^2                  * cos(a)^2
\\                  - 2* imag(I_GS_diff(k))*real(I_GS_diff(k)) * cos(a)*sin(a)
\\                  +    real(I_GS_diff(k))^2                  * sin(a)^2
\\                  )

Ika(k,a) =
{
  if(k==0, return(0));

    Ika_ref(k-1, a + Pi/4)
  + Ika_ref(k-1, a + 3*Pi/4)
  + 2*2^(k-1) * imag(I_GS_diff(k) * exp(I*(-a - Pi/4*k)))^2
  ;
}
check_exact(GS(0), 1/2)
check_exact(GS(1), 3/4+1/4*I)
check_exact(GS(1)-GS(0), 1/4+1/4*I)
check_exact(norm(GS(1)-GS(0)), 1/8)
check_exact(I_GS_norm(1), 1/8)
check_exact(I_GS_diff(1), 1/4 + 1/4*I)
check_equal(Ika_ref(1,Pi/2), 1/4)

check_equal(Ika(0,0), 0)
check_equal(Ika(0,Pi/2), 0)
check_equal(Ika(1,0), 0)
check_equal(Ika(1,Pi/2), 1/4)

check_equal(Ika(1,  Pi/4), 1/8)
check_equal(Ika(1,3*Pi/4), 1/8)
check_equal(Ika_ref(2,0), 1/2, "Ika_ref(2,0)")
check_equal(Ika_ref(2,Pi/2), 5/4, "Ika_ref(2,Pi/2)")
check_exact(I_GS_diff(2), -1/4 + 1/2*I)
check_exact(I_GS_norm(2), 5/16)
check_exact(I_GS_diff(2)*exp(I*Pi/4*2), -1/2 - 1/4*I)
check_exact(imag(I_GS_diff(2)*exp(I*-Pi/4*2)), 1/4)
check_exact(imag(I_GS_diff(2)*exp(I*-Pi/4*2))^2, 1/16)
check_exact(imag(I_GS_diff(2)*exp(I*-Pi/4*2))^2 * 2^2, 1/4)

\\ 1/8 + 1/8 + 1/4 == 1/2

check_equal(Ika(2,0), Ika_ref(2,0), "Ika(2,0)")
check_equal(Ika(2,Pi/2), Ika_ref(2,Pi/2))

check_equal(vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika(k,a))), \
            vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika_ref(k,a))))

\\  start                        end
\\    *              *------+-----*
\\    |              |                   GS(1) = 1/4 + 1/4*I
\\    |              |   +               GS(2) = 1/2 + 3/4*I
\\    |              |
\\    +          @   +                GS(2)-GS(1) = -1/4 + 1/2*I
\\    |              |                Ix = 2*(1/2)^2 = 1/2
\\    |   +          |                Iy = 2*(3/4)^2+2*(1/4)^2 = 5/4
\\    |              |
\\    *-------+------*


Ika(k,a) =
{
  if(k==0, return(0));

    Ika_ref(k-1, a + Pi/4)
  + Ika_ref(k-1, a + 3*Pi/4)
  + 2*2^(k-1) * imag(I_GS_rot(k) * exp(I*(-a)))^2
  ;
}
check_equal(vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika(k,a))), \
            vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika_ref(k,a))))

cis(a) = cos(a) + sin(a)*I;

Ika(k,a) =
{
  if(k==0, return(0));

    Ika_ref(k-1, a + Pi/4)
  + Ika_ref(k-1, a + 3*Pi/4)
  + 2*2^(k-1) * imag(I_GS_rot(k) * cis(-a))^2
  ;
}
check_equal(vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika(k,a))), \
            vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika_ref(k,a))))

Ika(k,a) =
{
  if(k==0, return(0));

    Ika_ref(k-1, a + Pi/4)
  + Ika_ref(k-1, a + 3*Pi/4)
  + 2*2^(k-1) * (imag(I_GS_rot(k))*cos(-a) + real(I_GS_rot(k))*sin(-a))^2
  ;
}
check_equal(vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika(k,a))), \
            vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika_ref(k,a))))

Ika(k,a) =
{
  if(k==0, return(0));

    Ika_ref(k-1, a + Pi/4)
  + Ika_ref(k-1, a + 3*Pi/4)
  + 2*2^(k-1) * (
                   imag(I_GS_rot(k))^2                      * cos(-a)^2
                 + 2* real(I_GS_rot(k))* imag(I_GS_rot(k))  * cos(-a)*sin(-a)
                 + real(I_GS_rot(k))^2                      * sin(-a)^2
                )
  ;
}
check_equal(vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika(k,a))), \
            vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika_ref(k,a))))

Ika(k,a) =
{
  if(k==0, return(0));

    Ika_ref(k-1, a + Pi/4)
  + Ika_ref(k-1, a + 3*Pi/4)
  + 2*2^(k-1) * (
                   imag(I_GS_rot(k))^2                      * cos(a)^2
                 - 2* real(I_GS_rot(k))* imag(I_GS_rot(k))  * cos(a)*sin(a)
                 + real(I_GS_rot(k))^2                      * sin(a)^2
                )
  ;
}
check_equal(vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika(k,a))), \
            vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika_ref(k,a))))

Ika(k,a) =
{
  if(k==0, return(0));

    Ika_ref(k-1, a + Pi/4)
  + Ika_ref(k-1, a + 3*Pi/4)
  + (
       imag(I_GS_rot_sqrt2k(k))^2                             * cos(a)^2
     - 2* real(I_GS_rot_sqrt2k(k))* imag(I_GS_rot_sqrt2k(k))  * cos(a)*sin(a)
     + real(I_GS_rot_sqrt2k(k))^2                             * sin(a)^2
    )
  ;
}
check_equal(vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika(k,a))), \
            vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika_ref(k,a))), \
            "Ika rot parts")

Ika(k,a) =
{
  if(k==0, return(0));

  + Ix_ref(k-1)   *cos(a + Pi/4)^2
  + Ixy_ref(k-1)  *cos(a + Pi/4)*sin(a + Pi/4)
  + Iy_ref(k-1)   *sin(a + Pi/4)^2
  + Ix_ref(k-1)   *cos(a + 3*Pi/4)^2
  + Ixy_ref(k-1)  *cos(a + 3*Pi/4)*sin(a + 3*Pi/4)
  + Iy_ref(k-1)   *sin(a + 3*Pi/4)^2
  + imag(I_GS_rot_sqrt2k(k))^2                             * cos(a)^2
  - 2* real(I_GS_rot_sqrt2k(k))* imag(I_GS_rot_sqrt2k(k))  * cos(a)*sin(a)
  + real(I_GS_rot_sqrt2k(k))^2                             * sin(a)^2
  ;
}
check_equal(vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika(k,a))), \
            vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika_ref(k,a))))

Ika(k,a) =
{
  if(k==0, return(0));

  + Ix_ref(k-1)   *( 1/2*cos(a)^2 - cos(a)*sin(a) + 1/2*sin(a)^2)
  + Ixy_ref(k-1)  *( 1/2*cos(a)^2                 - 1/2*sin(a)^2)
  + Iy_ref(k-1)   *( 1/2*cos(a)^2 + cos(a)*sin(a) + 1/2*sin(a)^2)
  + Ix_ref(k-1)   *( 1/2*cos(a)^2 + cos(a)*sin(a) + 1/2*sin(a)^2)
  + Ixy_ref(k-1)  *(-1/2*cos(a)^2                 + 1/2*sin(a)^2)
  + Iy_ref(k-1)   *( 1/2*cos(a)^2 - cos(a)*sin(a) + 1/2*sin(a)^2)
  + imag(I_GS_rot_sqrt2k(k))^2                             * cos(a)^2
  - 2* real(I_GS_rot_sqrt2k(k))* imag(I_GS_rot_sqrt2k(k))  * cos(a)*sin(a)
  + real(I_GS_rot_sqrt2k(k))^2                             * sin(a)^2
  ;
}
check_equal(vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika(k,a))), \
            vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika_ref(k,a))))

Ika(k,a) =
{
  if(k==0, return(0));

  + Ix_ref(k-1)   * cos(a)^2
  + Iy_ref(k-1)   * cos(a)^2
  + imag(I_GS_rot_sqrt2k(k))^2                             * cos(a)^2

  - 2* real(I_GS_rot_sqrt2k(k))* imag(I_GS_rot_sqrt2k(k))  * cos(a)*sin(a)

  + real(I_GS_rot_sqrt2k(k))^2                             * sin(a)^2
  + Ix_ref(k-1)   * sin(a)^2
  + Iy_ref(k-1)   * sin(a)^2
  ;
}
check_equal(vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika(k,a))), \
            vector(40,a,my(a=2*Pi/a); vector(10,k,my(k=k-1); Ika_ref(k,a))))

Ix(k) =
{
  if(k==0, return(0));
  + Ix_ref(k-1) 
  + Iy_ref(k-1) 
  + imag(I_GS_rot_sqrt2k(k))^2 
  ;
}
check_equal(vector(10,k,my(k=k-1); Ix(k)), \
            vector(10,k,my(k=k-1); Ix_ref(k)), "Ix")

Ix(k) =
{
  if(k==0, return(0));
  Iz_ref(k-1) + imag(I_GS_rot_sqrt2k(k))^2 
  ;
}
check_equal(vector(10,k,my(k=k-1); Ix(k)), \
            vector(10,k,my(k=k-1); Ix_ref(k)), "Ix")

Iy(k) =
{
  if(k==0, return(0));
  Iz_ref(k-1) + real(I_GS_rot_sqrt2k(k))^2 
  ;
}
check_equal(vector(10,k,my(k=k-1); Iy(k)), \
            vector(10,k,my(k=k-1); Iy_ref(k)), "Iy")

Iz(k) =
{
  if(k==0, return(0));
  2*Iz_ref(k-1) + norm(I_GS_rot_sqrt2k(k))
  ;
}
check_equal(vector(10,k,my(k=k-1); Iz(k)), \
            vector(10,k,my(k=k-1); Iz_ref(k)), "Iz")

Ixy(k) =
{
  - 2* real(I_GS_rot_sqrt2k(k))* imag(I_GS_rot_sqrt2k(k))
  ;
}
check_equal(vector(10,k,my(k=k-1); Ixy(k)), \
            vector(10,k,my(k=k-1); Ixy_ref(k)), "Ixy")

Ixy(k) =
{
  - 2
  * (
     3/10 * 2^k + real( (1/5 - 1/10*I) * (-I)^k)
    )
  * (
     1/10 * 2^k + imag( (1/5 - 1/10*I) * (-I)^k)
    )
  ;
}
check_equal(vector(10,k,my(k=k-1); Ixy(k)), \
            vector(10,k,my(k=k-1); Ixy_ref(k)), "Ixy")

Ixy(k) =
{
  - 2
  * (
     3/10 * 2^k + vector_modulo([1/5, -1/10, -1/5, 1/10],k)
    )
  * (
     1/10 * 2^k + vector_modulo([-1/10, -1/5, 1/10, 1/5],k)
    )
  ;
}
check_equal(vector(10,k,my(k=k-1); Ixy(k)), \
            vector(10,k,my(k=k-1); Ixy_ref(k)), "Ixy")

Ixy(k) =
{
  -3/50 * 4^k
  + vector_modulo([1/50, 7/50, -1/50, -7/50],k) * 2^k
  + 1/25*(-1)^k
  ;
}
check_equal(vector(10,k,my(k=k-1); Ixy(k)), \
            vector(10,k,my(k=k-1); Ixy_ref(k)), "Ixy")


print(vector(8,k,k-=1; Ixy_ref(k)))

\\-----------------------------------------------------------------------------
\\ Iz(k)

Iz(k) =
{
  if(k==0, return(0));
  2*Iz_ref(k-1) + norm(I_GS_rot_sqrt2k(k))
  ;
}
check_equal(vector(10,k,my(k=k-1); Iz(k)), \
            vector(10,k,my(k=k-1); Iz_ref(k)), "Iz")

check_exact(vector(100,k, 1/2^k*norm(I_GS_rot_sqrt2k(k))), \
            vector(100,k, I_GS_norm_ref(k))) /* k>=1 */

Iz(k) =
{
  if(k==0, return(0));
  2*Iz(k-1) + 2^k*I_GS_norm_ref(k)
  ;
}
check_equal(vector(10,k,my(k=k-1); Iz(k)), \
            vector(10,k,my(k=k-1); Iz_ref(k)), "Iz")

gI_GS_norm_2k(x) =
{
  + 1/10*1/(1 - 4*x)
  + 1/20*1/(1 - x)
  + (-2*x + 1)/(40*x^2 + 10)
;
}
\\ print(vector(40,k,my(k=k-1); 2^k*I_GS_norm(k)));
check_equal(gf_terms(gI_GS_norm_2k(x),10), \
            vector(10,k,my(k=k-1); 2^k*I_GS_norm(k)), "gI_GS_norm_2k")

gIz(x) =
{
  1/5  *  1/(1 - 4*x)

  + 1/4  * -1/(1 - 2*x)
  + 1/10 *  1/(1 + 4*x^2)

  + 1/20   * -1/(1 - x)
  ;
}
\\ print(gf_terms(gIz(x),5));
\\ print(vector(50,k,my(k=k-1); Iz(k)))

check_equal(gf_terms(gIz(x),10), \
            vector(10,k,my(k=k-1); Iz_ref(k)), "gIz")

{
  check_exact(
              x
              * (1/4 + 3*x^2 - 4*x^3)
              / (1-2*x)
              / (1 - 5*x + 8*x^2 - 20*x^3 + 16*x^4)
              ,
              gIz(x))
}
{
  check_exact(
             (1/4 - x + 2*x^2 - 2*x^3)
              / (1 - 5*x + 8*x^2 - 20*x^3 + 16*x^4)
              ,
              gI_GS_norm_2k(x))
}
{
  check_exact(
              1/(1-2*x) * (gI_GS_norm_2k(x) - I_GS_norm(0))
              ,
              gIz(x))
}

Iz(k) =
{
  1/5 * 4^k
  - 1/20 * vector_modulo([3, 5, 7, 5],k) * 2^k
  - 1/20 
  ;
}
check_equal(vector(10,k,my(k=k-1); Iz(k)), \
            vector(10,k,my(k=k-1); Iz_ref(k)), "Iz")

\\ print(vector(8,k,my(k=k-1); Iz_ref(k)))

\\-----------------------------------------------------------------------------

Iy(k) =
{
  if(k==0, return(0));
  Iz(k-1) + real(I_GS_rot_sqrt2k(k))^2 
  ;
}
check_equal(vector(10,k,my(k=k-1); Iy(k)), \
            vector(10,k,my(k=k-1); Iy_ref(k)), "Iy")

\\ print(vector(50,k,my(k=k-1); Iy(k)))

gIy(x) =
{
  + 7/50 * 1/(1 - 4*x)

  + 1/8*(-1)/(1 - 2*x)
  + (3/25 - 1/50*x)/(1 + 4*x^2)

  + 1/40*(-1)/(1 - x)
  + 1/200*(3)/(1 + x)

  - 1/8
  ;
}
check_equal(gf_terms(gIy(x),100), \
            vector(100,k,my(k=k-1); Iy(k)), "gIy")

\\ for(i=1,#v, print(v[i]/2^i))

{
  check_equal(gf_terms(+ 1/8*(-1)/(1 - 2*x)
                       + (3/25 - 1/50*x)/(1 + 4*x^2)
                       ,100),
              vector(100,k,my(k=k-1);
                     - 1/200 * vector_modulo([1, 27, 49, 23],k) * 2^k))
}

Iy(k) =
{
  7/50 * 4^k
  - 1/200 * vector_modulo([1, 27, 49, 23],k) * 2^k
  - vector_modulo([1/100, 1/25],k)
  + if(k==0, - 1/8)
  ;
}
\\ print(gf_terms(gIy(x),5));
\\ print(vector(5,k,my(k=k-1); Iy(k)))
check_equal(vector(10,k,my(k=k-1); Iy(k)), \
            vector(10,k,my(k=k-1); Iy_ref(k)), "Iy powers")
check_equal(gf_terms(gIy(x),100), \
            vector(100,k,my(k=k-1); Iy(k)), "gIy")


\\-----------------------------------------------------------------------------

Ix(k) =
{
  if(k==0, return(0));
  Iz(k-1) + imag(I_GS_rot_sqrt2k(k))^2 
  ;
}
check_equal(vector(10,k,my(k=k-1); Ix(k)), \
            vector(10,k,my(k=k-1); Ix_ref(k)), "Ix")

\\ print(vector(50,k,my(k=k-1); Ix(k)))

gIx(x) =
{
     + 1/50*(3)/(1 - 4*x)

     + 1/8*(-1)/(1 - 2*x)
     + 1/200*(-2 + I)/(1 + 2*I*x)
     + 1/200*(-2 - I)/(1 - 2*I*x)

     + 1/40*(-1)/(1 - x)
     + 1/200*(-3)/(1 + x)

     + 1/8
  ;
}
check_equal(gf_terms(gIx(x),100), \
            vector(100,k,my(k=k-1); Ix(k)), "gIx")

\\ for(i=1,#v, print(v[i]/2^i))

{
  check_equal(gf_terms(+ 1/8*(-1)/(1 - 2*x)
                       + 1/200*(-2 + I)/(1 + 2*I*x)
                       + 1/200*(-2 - I)/(1 - 2*I*x)
                       ,100),
              vector(100,k,my(k=k-1);
                     -1/200 * vector_modulo([29, 23, 21, 27 ],k) * 2^k))
}

Ix(k) =
{
  3/50 * 4^k
  - 1/200 * vector_modulo([29, 23, 21, 27 ],k) * 2^k
  - vector_modulo([1/25, 1/100],k)
  + if(k==0, + 1/8)
  ;
}
\\ print(gf_terms(gIx(x),5));
\\ print(vector(5,k,my(k=k-1); Ix(k)))
check_equal(vector(10,k,my(k=k-1); Ix(k)), \
            vector(10,k,my(k=k-1); Ix_ref(k)), "Ix powers")
check_equal(gf_terms(gIx(x),100), \
            vector(100,k,my(k=k-1); Ix(k)), "gIx")


\\ for(k=0,7,printtex(Ix(k),","))
\\ for(k=0,7,printtex(Iy(k),","))

\\-----------------------------------------------------------------------------
\\ Iz complete the square

Iz_by_powers = Iz;

Iz(k) =
{
  1/5 * (
         4^k
         - 1/4 * vector_modulo([3, 5, 7, 5],k) * 2^k
         - 1/4
        )
  ;
}
check_equal(vector(100,k,my(k=k-1); Iz(k)), \
            vector(100,k,my(k=k-1); Iz_by_powers(k)), "Iz")

