\\ Copyright 2015, 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

conj_real(z) = -real(z) + I*imag(z);
real_imag_transpose(z) = imag(z) + I*real(z);


\\-----------------------------------------------------------------------------

area_of_row_integrals(ymin,ymax, xmin,xmax) = {
  my(poly = intformal(1,'x));
  poly = subst(poly, 'x, xmax) - subst(poly, 'x, xmin);
  poly = intformal(poly,'y);
  subst(poly, 'y, ymax) - subst(poly, 'y, ymin);
}
centroid_of_row_integrals(ymin,ymax, xmin,xmax) = {
  my(poly = intformal('x+'y*I,'x));
  poly = subst(poly, 'x, xmax) - subst(poly, 'x, xmin);
  poly = intformal(poly,'y);
  subst(poly, 'y, ymax) - subst(poly, 'y, ymin);
}

\\
\\                          2/3, 1/3
\\                            P6= ----- P7=1, 1/3
\\                                          \
\\       P5=-1/6, 1/6                       P8=7/6, 1/6
\\      /                                    |
\\    P4=-1/3, 0                            P9=7/6, 0
\\     |
\\     |
\\    P3=-1/3, -1/3
\\      \                            P10=5/6, -1/2
\\       \                          /
\\        P2=0, -2/3 ---------P1=2/3, -2/3

check_exact( 1/(1/6)*(2/3-(-1/6)),  5);     /* slope */
check_exact( 1/(1/2)*(7/6-5/6),  2/3);   /* slope */
my(y=-2/3); check_exact( 0-(y-(-2/3)),  0);   /* lower left 1 */
my(y=-1/3); check_exact( 0-(y-(-2/3)),  -1/3);   /* lower left 1 */
my(y=-2/3); check_exact( (y-(-2/3))+2/3,  2/3);   /* lower right 1 */
my(y=-1/2); check_exact( (y-(-2/3))+2/3,  5/6);   /* lower right 1 */
my(y=-1/2); check_exact( (y-(-1/2))/(1/2)*(7/6-5/6)+5/6,  5/6);   /* lower right 2 */
my(y=0); check_exact( (y-(-1/2))/(1/2)*(7/6-5/6)+5/6,  7/6);   /* lower right 2 */
my(y=1/6); check_exact( (y-1/6)/(1/6)*(2/3-(-1/6))-1/6,  -1/6);  /* upper slope */
my(y=1/3); check_exact( (y-1/6)/(1/6)*(2/3-(-1/6))-1/6,  2/3);   /* upper slope */
my(y=1/3); check_exact( 7/6-(y-1/6),  1);   /* upper 45 */

check_exact( geom_area_of_polygon([-1/3, -1/3-1/3*I, -2/3*I, 2/3-2/3*I, 5/6-1/2*I, 7/6 ]),  19/24);  /* upper */
check_exact( geom_area_of_polygon([7/6, 7/6+1/6*I, 1+1/3*I, 2/3+1/3*I, -1/6+1/6*I, -1/3 ]),  3/8);  /* lower */
check_exact( 19/24+3/8,  HAf);

{
  /* upper 1 */
  check_exact( area_of_row_integrals(0,1/6, -1/3+y,7/6),
              geom_area_of_polygon([-1/3, 7/6, 7/6+1/6*I, -1/6+1/6*I]));
}
{
  /* upper 2 */
  check_exact( area_of_row_integrals(1/6,1/3, (y-1/6)/(1/6)*(2/3-(-1/6))-1/6, 7/6-(y-1/6)),
              geom_area_of_polygon([-1/6+1/6*I, 7/6+1/6*I, 1+1/3*I, 2/3+1/3*I]));
}

{my(v=[
  /* lower left */
  [ -2/3, -1/3,   0-('y-(-2/3)), 2/3 ],  /* -45deg */
  [ -1/3, 0,      -1/3,         2/3 ],  /* vertical */
  /* lower right */
  [ -2/3, -1/2,  2/3, ('y-(-2/3))+2/3  ], /* +45deg */
  [ -1/2, 0,     2/3, ('y-(-1/2))/(1/2)*(7/6-5/6)+5/6 ], /* slope */
  /* upper */
  [ 0, 1/6,       -1/3+'y,     7/6   ],  /* +45, vertical */
  [ 1/6,1/3, ('y-1/6)/(1/6)*(2/3-(-1/6))-1/6,  /* slope */
             7/6-('y-1/6) ]  /* -45deg */
  ]);
for(i=1,#v,
    v[i][1] -= imag(HGf);
    v[i][2] -= imag(HGf);
    v[i][3] -= real(HGf);
    v[i][4] -= real(HGf);
    v[i][3] = subst(v[i][3], 'y,'y+imag(HGf));
    v[i][4] = subst(v[i][4], 'y,'y+imag(HGf));
   );
hull_fractal_area_by_integrals =
  sum(i=1,#v,
      area_of_row_integrals(v[i][1],v[i][2],
                            v[i][3],v[i][4]));
hull_fractal_centroid_by_integrals =
  sum(i=1,#v,
      centroid_of_row_integrals(v[i][1],v[i][2],
                                v[i][3],v[i][4]));
hull_fractal_I_by_integrals =
  sum(i=1,#v,
      geom_inertia_of_intformal_rows(v[i][1],v[i][2],
                                     v[i][3],v[i][4]));
}
check_exact( geom_area_of_polygon(hull_fractal),  HAf);
check_exact( hull_fractal_area_by_integrals,  HAf);
check_exact( hull_fractal_I_by_integrals,  hull_fractal_I);
check_exact( hull_fractal_centroid_by_integrals,  0);
check_exact( geom_centroid_of_polygon(hull_fractal),  HGf);

\\-----------------------------------------------------------------------------

my(k=64); poly = hull(k)/b^k;
G = geom_centroid_of_polygon(poly);
if(abs(G) < 1e20, print("centroid ",G));
poly = apply(z->z-G, poly);

print("hull inertia");
m = geom_inertia_of_polygon(poly);
geom_inertia__print(m);

d = geom_inertia_principal_xydir(m);
d=d[1]+d[2]*I;
print("d " d);
print("d slope " imag(d)/real(d), " ", imag(d)/real(d)*1.0, "   cf 3/4");
print("d sqrt " sqrt(d));
alpha_max = geom_inertia_principal_xyangle(m);
alpha_min = alpha_max + Pi/2;
print("degrees min ",alpha_min*180/Pi-180,"   max ",alpha_max*180/Pi);
print("radians min ",alpha_min-Pi,"   max ",alpha_max);
print("I       min ",geom_inertia_I_at_xyangle(m,alpha_min))
print("I       max ",geom_inertia_I_at_xyangle(m,alpha_max))

\\-----------------------------------------------------------------------------

print();
print("of fractal hull limit");

poly = hull_fractal;
G = geom_centroid_of_polygon(poly);
check_exact(G, HGf);
if(abs(G) < 1e20, print("centroid ",G));
poly = apply(z->z-G, poly);

m = geom_inertia_of_polygon(poly);
check_exact(m, hull_fractal_I);
geom_inertia__print(m);

d = geom_inertia_principal_xydir(m);
d=d[1]+d[2]*I;
print("d " d);
print("d slope " imag(d)/real(d), " ", imag(d)/real(d)*1.0, "   cf 3/4");
print("d sqrt " sqrt(d));
alpha_max = geom_inertia_principal_xyangle(m);
alpha_min = alpha_max + Pi/2;
print("degrees min ",alpha_min*180/Pi,"   max ",alpha_max*180/Pi);
print("radians min ",alpha_min,"   max ",alpha_max);
print("I       min ",geom_inertia_I_at_xyangle(m,alpha_min))
print("I       max ",geom_inertia_I_at_xyangle(m,alpha_max))


