\\ Copyright 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

\\ Similar: rationals devel/A142151-bit-masks.gp

default(recover,0);
default(strictargs,1);

check_count = 0;
check_equal(got,want,name="") =
{
  check_count++;
  if(got != want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

gf_terms(g,n) = {
  if(g==0,return(vector(n)));
  my(x = variable(g),
     zeros = min(n,valuation(g,x)),
     v = Vec(g + O(x^n)));
  if(zeros>=0, concat(vector(zeros,i,0), v),
               v[-zeros+1 .. #v]);
}


\\-----------------------------------------------------------------------------

\\ numbers with odd num 0s
A059009(n) =
{
  n>=0 || error();
  if(n==0,0, n<<1 + bitnegimply(1, logint(n,2) - hammingweight(n)));
}
A010060(n) =
{
  n>=0 || error();
  hammingweight(n)%2;
}
A030300(n) =
{
  n>=0 || error();
  if(n, !(logint(n,2)%2));
}
{
  my(want=OEIS_samples("A030300")); \\ OFFSET=1
  check_equal(vector(#want,n, A030300(n)),
              want,
              "A030300 samples");
}

A000120(n) = hammingweight(n);
{
  my(want=OEIS_samples("A000120")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A000120(n)),
              want,
              "A000120 samples");
}
gA000120(x,limit) = sum(n=0,limit-1,A000120(n)*x^n) + O(x^limit);
{
  check_equal(gA000120(x,256),
              1/(1-x) * sum(k=0,8, x^(2^k)/(1+x^(2^k))),
              "gA000120() by sum");
}


\\-----------------------------------------------------------------------------
\\ A004718 - Per Norgard's infinity sequence

infinity(n) =
{
  \\ high to low, +1 for 1-bit, negate for 0-bit
  n>=0 || error();
  my(ret=0);
  forstep(i=if(n,logint(n,2)),0,-1, 
     if(bittest(n,i), ret++, ret=-ret));
  ret;
}
A004718=infinity;
{
  my(want=OEIS_samples("A004718")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A004718(n)),
              want,
              "infinity() A004718 samples");

  \\ want=OEIS_bfile_samples("A004718");
  \\ print("b004718 size ",#want);
  \\ check_equal(vector(#want,n,n--; A004718(n)),
  \\             want,
  \\             "infinity() b004718.txt samples");

  check_equal(vector(1024,n,n--;
                     \\ low to high, add s at each 1-bit, negate s each 0-bit
                     my(s=1,ret=0);
                     for(i=0,if(n,logint(n,2)), 
                        if(bittest(n,i), ret+=s, s=-s));
                     ret),
              vector(1024,n,n--; infinity(n)),
              "infinity() low to high");

  \\ a(2n) = -a(n), a(2n+1) = a(n) + 1
  check_equal(vector(1024,n,n--; infinity(2*n)),
              vector(1024,n,n--; -infinity(n)));
  check_equal(vector(1024,n,n--; infinity(2*n+1)),
              vector(1024,n,n--; infinity(n) + 1));
}

ginfinity(x,k) =
{
  sum(k=0,k-1,
      x^(2^k) * prod(l=0,k-1, x^(2^l)-1)
      / (1-x^(2*2^k)) );
}
{
  check_equal(Vec(prod(l=0,6, x^(2^l)-1)),     \\ A106400 Thue-Morse signed
              vector(128,n,n--; (-1)^hammingweight(n)),
              "prod part signed Thue-Morse");

  check_equal(ginfinity(x,10) + O(x^1024), 
              sum(n=0,1023, infinity(n)*x^n),
              "ginfinity()");

  \\  F(x) = (1-x)*F(x^2) + x/(1-x^2)
  \\ with correction to x-1
  check_equal(ginfinity(x,8),
              ginfinity(x^2,8)*(x-1) + x/(1-x^2) + O(x^16), 
              "ginfinity() satisfies");

  check_equal(gf_terms(x/(1-x^2), 256),
              vector(256,n,n--; n%2==1),
              "x/(1-x^2) = 1 at odd");

  check_equal(ginfinity(x,6),
              x/(1-x^2) + (x-1)*( 
              x^2/(1-x^4) + (x^2-1)*(  
              x^4/(1-x^8) + (x^4-1)*( 
              x^8/(1-x^16) + (x^8-1)*(  
              x^16/(1-x^32) + (x^16-1)*( 
              x^32/(1-x^64) + (x^32-1)*( 
              0 )))))),
              "ginfinity() nested");

}

\\-----------------------------------------------------------------------------

even(n) = my(t=1,ret=0); \
  for(i=0,if(n,logint(n,2)), if(bittest(n,i), ret+=t, t=!t)); ret;
odd(n) = my(t=0,ret=0); \
  for(i=0,if(n,logint(n,2)), if(bittest(n,i), ret+=t, t=!t)); ret;

\\ vector(50,n,n++; even(n))
\\ vector(50,n,n++; odd(n))
\\ not in OEIS: 0, 2, 1, 1, 0, 3, 0, 2, 1, 2, 2, 1, 0, 4, 1, 1, 0, 3, 1, 2, 1
\\ not in OEIS: 1, 2, 0, 1, 0, 1, 1, 0, 2, 3, 0, 0, 1, 2, 0, 1, 1, 2, 1

\\ vector(20,n,n++; even(n+1)-even(n))
\\ vector(20,n,n++; odd(n+1)-odd(n))
\\ not in OEIS: 2, -1, 0, -1, 3, -3, 2, -1, 1, 0, -1, -1, 4, -3, 0, -1, 3, -2, 1, -1
\\ not in OEIS: -1, 0, 1, 1, -2, 1, -1, 1, 0, -1, 2, 1, -3, 0, 1, 1, -2, 1, 0, 1

\\ vector(20,n,n++; hammingweight(n)%2==0)
\\ vector(20,n,n++; even(n)%2)
\\ vector(30,n,n++; odd(n)%2)
\\ not in OEIS: 0, 0, 1, 1, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 1, 0, 1, 1, 0
\\ not in OEIS: 1,0,0,1,0,0,1,0,1,1,0,0,1,0,0,1,0,0,1,1,0,1,0,0,1,0,0,1,0,0
\\ not A242718 darts
\\ not A282317 cube-free
\\ not A286691 partial match

{
  check_equal(vector(5000,n,n--; infinity(n)),
              vector(5000,n,n--; even(n) - odd(n)));

  check_equal(vector(5000,n,n--; even(n)),
              vector(5000,n,n--; odd(2*n)));
  check_equal(vector(5000,n,n--; even(2*n)),
              vector(5000,n,n--; odd(n)));

  check_equal(vector(5000,n,n--; even(2*n)),
              vector(5000,n,n--; hammingweight(n) - even(n)));
  check_equal(vector(5000,n,n--; even(2*n+1)),
              vector(5000,n,n--; even(n) + 1),
              "even() of odd");

  check_equal(vector(5000,n,n--; odd(2*n)),
              vector(5000,n,n--; hammingweight(n) - odd(n)));
  check_equal(vector(5000,n,n--; odd(2*n+1)),
              vector(5000,n,n--; odd(n)),
              "odd() of odd");
}
{
  my(n   =from_binary(1101011100));
  check_equal(n, 860);
  check_equal(even(n), 5);
}
{
  my(n   =from_binary(1001110100));
  check_equal(n, 628);
  check_equal(odd(n), 4);
}

geven(x,limit) = sum(n=0,limit-1,even(n)*x^n) + O(x^limit);
godd(x,limit)  = sum(n=0,limit-1,odd(n)*x^n)  + O(x^limit);
{
  my(limit=256);
  check_equal(geven(x,limit),
              (x-1)*geven(x^2,limit) + gA000120(x^2,limit) + x/(1-x^2));

  check_equal(godd(x^2,limit),
              geven(x,limit) - x*geven(x^2,limit) - x/(1-x^2));
  check_equal(geven(x^2,limit),
              godd(x,limit)  - x*godd(x^2,limit));

  check_equal(geven(x,1024),
              sum(k=0,10,
                  x^(2^k) * 1/2*((1-x^(2^k))/(1-x) + prod(j=0,k-1, x^(2^j)-1))
                  / (1-x^(2*2^k))),
              "geven() by terms 1");
  check_equal(geven(x,1024),
              1/2* sum(k=0,10,
                  x^(2^k) /(1-x) / (1 + x^(2^k))     \\ total is hammingweight
                  + x^(2^k) * prod(j=0,k-1, x^(2^j)-1) / (1-x^(2*2^k))),
              "geven() by terms 2");
  check_equal(geven(x,1024),
              1/(1-x)
              * sum(k=0,10,
                    x^(2^k) * 1/2*(1-x^(2^k) + (1-x)*prod(j=0,k-1, x^(2^j)-1))
                    /(1-x^(2*2^k))),
              "geven() by terms 3");

  check_equal(godd(x,1024),
              sum(k=0,10,
                  x^(2^k) * 1/2*((1-x^(2^k))/(1-x) - prod(j=0,k-1, x^(2^j)-1))
                  / (1-x^(2*2^k))),
              "godd() by terms");


  check_equal(prod(j=0,2, x^(2^j)-1),
              -1 + x + x^2 - x^3 + x^4 - x^5 - x^6 + x^7,
              "geven(),godd() product part 8");
  check_equal(prod(j=0,2, x^(2^j)-1),
              sum(n=0,7, if(hammingweight(n)%2==0, -1, 1) * x^n),
              "geven(),godd() product part 8 vs hammingweight%2");
  check_equal(prod(j=0,3, x^(2^j)-1),
              sum(n=0,15, if(hammingweight(n)%2==0, 1, -1) * x^n),
              "geven(),godd() product part 16 vs hammingweight%2 opposite");
}

\\-----------------------------------------------------------------------------

infinity_vector(n) =
{
  n>=0 || error();
  my(v=binary(n),s=1);
  forstep(i=#v,1,-1,     \\ low to high
     if(v[i], v[i]=s, s=-s));
  v;
}
{
  check_equal(vector(5000,n,n--; vecsum(infinity_vector(n))),
              vector(5000,n,n--; infinity(n)),
              "infinity_vector() sum");
}

infinity_balanced_ternary(n) = fromdigits(infinity_vector(n),3);
\\ vector(20,n,n++; infinity_balanced_ternary(n))
\\ not in OEIS: -3,4,9,-8,-12,13,-27,28,24,-23,36,-35,-39,40,81,-80,-84,85,-72,73
{
  check_equal(vector(5000,n,n--; my(b=infinity_balanced_ternary(n) % 3);
                     b==0 || b==1),
              vector(5000,n,n--; 1),
              "infinity_balanced_ternary() == 0 or 1 mod 3");
}

infinity_signed_binary(n) = fromdigits(infinity_vector(n),2);
\\ vector(20,n,n++; infinity_signed_binary(n))
\\ not in OEIS: -2,3,4,-3,-6,7,-8,9,6,-5,12,-11,-14,15,16,-15,-18,19,-12,13
\\

{
  check_equal(infinity_signed_binary(1), 1);
  check_equal(infinity_signed_binary(2), -2);
  check_equal(infinity_signed_binary(3), 3);
  check_equal(infinity_signed_binary(5), -4+1);

  my(m=Map(), bad=0);
  for(n=0,2048,
     my(b=infinity_signed_binary(n));
     if(mapisdefined(m,b), bad++, mapput(m,b,n)));
  check_equal(bad, 0,
              "infinity_signed_binary() no duplicates");

  \\ no occur when high 1-bit contradicts sign
  \\ all and only 3n+2 positives and -(3n+1) negatives, so all signed 3n+2
  \\ 2,5,8,11,14,17,20,23,26,29,32,35,38,41,44,47,50,53,56,59,62,65,68,71,74,77,80,83,86,89,92,95,98,101,104,107,110,113,116,119,122,125,128
  \\ neg
  \\ 1,4,7,10,13,16,19,22,25,28,31,34,37,40,43,46,49,52,55,58,61,64,67,70,73,76,79,82,85,88,91,94,97,100,103,106,109,112,115,118,121,124,127
  \\ for(i=0,128,
  \\    if(!mapisdefined(m,-i),
  \\       print1(i,",")));
  \\ print();
}

\\-----------------------------------------------------------------------------
\\ Num Zeros in N

num_zeros(n) = if(n==0,0, logint(n,2)+1 - hammingweight(n));
{
  check_equal(vector(1024,n,n--; my(ret=0); while(n, if(n%2==0,ret++); n>>=1); ret),
              vector(1024,n,n--; num_zeros(n)));
}

A023416(n) =
{
  \\ A023416 wrongly takes n=0 as one zero
  if(n==0,1, num_zeros(n));
}
{
  my(want=OEIS_samples("A023416")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A023416(n)),
              want,
              "A023416 num_zeros() samples");
}

\\-----------------------------------------------------------------------------
\\ A059448  parity num 0s
\\   0, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1,
\\ n=1  2  3  4  5

A059448(n) =
{
  n>=1 || error();
  num_zeros(n) % 2;
}
{
  my(want=OEIS_samples("A059448")); \\ OFFSET=1
  check_equal(vector(#want,n, A059448(n)),
              want,
              "A059448 num_zeros() samples");

  \\ formulas in A059448
  my(a=A059448);
  check_equal(vector(1024,n, a(2*n)),
              vector(1024,n, 1 - a(n)),
              "A059448 even");
  check_equal(vector(1024,n, a(2*n+1)),
              vector(1024,n, 1 - a(2*n)),
              "A059448 odd, pairs of opposites");
  check_equal(vector(10,k, vector(2^k,n,n+=2^k-1; a(n))),
              vector(10,k, vector(2^k,n,n+=2^k-1; 1 - a(n-2^(k-1)))),
              "A059448 in 2^k blocks flip second highest");
  check_equal(vector(1024,n, a(n)),
              vector(1024,n, A023416(n) % 2),
              "A059448 = A023416 mod 2");
  check_equal(vector(1024,n, a(n)),
              vector(1024,n, A059009(n) - 2*n),
              "A059448 from A059009");

  \\ Shown as |A010060(n) - A030300(n-1)| instead.
  check_equal(vector(4096,n, a(n)),
              vector(4096,n, abs(A010060(n) - A030300(n))),
              "A059448 from Thue-Morse A010060 and flip A030300");
}


\\-----------------------------------------------------------------------------
\\ A059009  numbers with odd num zeros
\\ taking 0 as one 0 digit

{
  my(want=OEIS_samples("A059009")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A059009(n)),
              want,
              "A059009 odd num 0s");

  \\ formulas by Ralf Stephan in A059009
  my(a=A059009);
  check_equal(a(0), 0);
  check_equal(vector(1024,n, a(2*n)),
              vector(1024,n, 6*n + 1 - a(n)),
              "A059009 even");
  check_equal(vector(1024,n,n--; a(2*n+1)),
              vector(1024,n,n--; a(n) + 2*n + 2),
              "A059009 odd");
  check_equal(vector(1024,n, a(n)),
              vector(1024,n, 2*n + (1/2)*(1-(-1)^A023416(n))),
              "A059009 using A023416");
  check_equal(vector(1024,n, a(n)),
              vector(1024,n, 2*n + A059448(n)),
              "A059009 using A059448");
}

\\-----------------------------------------------------------------------------
\\ A059010  numbers with even num non-leading zeros

A059010(n) =
{
  n>=0 || error();
  if(n==0,1, 2*n + (logint(n,2) - hammingweight(n))%2);
}
{
  my(want=OEIS_samples("A059010")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A059010(n)),
              want,
              "A059010 even num 0s");

  \\ formulas by Ralf Stephan in A059010
  my(a=A059010);
  check_equal(a(0), 1);
  check_equal(vector(1024,n, a(2*n)),
              vector(1024,n, 6*n + 1 - a(n)),
              "A059010 even");
  check_equal(vector(1024,n,n--; a(2*n+1)),
              vector(1024,n,n--; a(n) + 2*n + 2),
              "A059010 odd");
  check_equal(vector(1024,n, a(n)),
              vector(1024,n, 2*n+1 - (1/2)*(1 - (-1)^A023416(n))),
              "A059010 using A023416");
  check_equal(vector(1024,n, a(n)),
              vector(1024,n, 2*n + 1 - A059448(n)),
              "A059010 using A059448");
}

\\-----------------------------------------------------------------------------
\\ A298952  1 - parity of num 0-bits in n
\\   1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0,
\\ n=0  1  2  3  4  5

A298952(n) = if(n==0,1, (logint(n,2) - hammingweight(n)) % 2);
{
  my(want=OEIS_samples("A298952")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A298952(n)),
              want,
              "A298952() vs samples");

  check_equal(A298952(0), 1);  \\ even, no 0s at all
  check_equal(A298952(1), 1);  \\ even, no 0s at all
  check_equal(A298952(2), 0);  \\ odd
  check_equal(A298952(4), 1);  \\ even 
  check_equal(A298952(5), 0);  \\ odd

  check_equal(vector(2^12,n,n--; infinity_signed_binary(n) >= 0),
              vector(2^12,n,n--; A298952(n)),
              "A298952 = infinity_signed_binary(n) >= 0");

  my(a=A298952);
  check_equal(vector(1024,n,n--; a(2*n+1)),
              vector(1024,n,n--; a(n)),
              "A298952 extra low 1-bit");
  check_equal(vector(1024,n,n--; a(2*n)),
              vector(1024,n,n--; if(n==0,1, 1-a(n))),
              "A298952 extra low 0-bit");
}

\\ construction in A298952
A298952_vector(len) =
{
  my(v=vector(len));
  v[0 +1] = 1;
  for(n=0,#v-1,
     my(t = 2*n + v[n +1]);
     if(t+1<=#v, v[t +1]=1));
  v;
}
{
  check_equal(A298952_vector(2^12),
              vector(2^12,n,n--; A298952(n)),
              "A298952_vector() vs A298952()");
}

mu_expand(v) = concat(apply(t->[1-t,t], v));
{
  \\ Michel Dekking in A298952
  my(v=A298952_vector(2^10),
     a2onwards = v[3..#v],
     a1onwards = v[2..#v],
     m=mu_expand(a1onwards));
  check_equal(a2onwards, m[1..#a2onwards],
              "A298952 under mu_expand()");
}

{
  my(want=A298952_vector(2^10));
  check_equal(vector(#want,n,n--; num_zeros(n)%2==0),
              want,
              "A298952 num_zeros even");
}

\\ select(n->num_zeros(n)%2==0, [1..64])
\\ A059010
\\ select(n->num_zeros(n)%2==1, [1..64])
\\ A059009

{
  my(c=OEIS_samples("A298952"),
     u=select(n->c[n +1]==1, [0..#c-1]),
     v=OEIS_samples("A059010"));
  v=select(t->t<=#c-1, v);
  v=concat(0,v);
  check_equal(u, v,
              "A298952 char func of A059010 except 0");
}
{
  my(want=select(n->A298952(n)==0, [1..256]));
  check_equal(vector(#want,n, A059009(n)),
              want,
              "A298952 indices of 1 is A059009");
}
{
  my(want=select(n->A298952(n)==1, [1..256]));
  check_equal(vector(#want,n,n--; A059010(n)),
              want,
              "A298952 indices of 1 is A059010");
}


\\--------------------
\\ A298952 cumulative

\\ vector(20,n,n++; sum(i=0,n-1, A298952(i)))
\\ vector(20,n,n++; sum(i=1,n-1, A298952(i)))
\\ not in OEIS: 2, 2, 3, 4, 4, 4, 5, 5, 6, 7, 7, 8, 8, 8, 9, 10, 10, 10, 11, 11
\\ not in OEIS: 1, 1, 2, 3, 3, 3, 4, 4, 5, 6, 6, 7, 7, 7, 8, 9, 9, 9, 10, 10
{
  check_equal(vector(128,n,n--; A298952(2*n) + A298952(2*n+1)),
              vector(128,n,n--; if(n==0,2, 1)),
              "A298952 pairs of terms");

  check_equal(vector(128,n,n--; n\2 + (n>=2) + if(n%2, 1-(num_zeros(n-1)%2))),
              vector(128,n,n--; sum(i=0,n-1, A298952(i))),
              "A298952 cumulative, n/2 up or down");

  check_equal(vector(128,n,n--; sum(i=0,n-1, A298952(i))) -
              vector(128,n,n--; n\/2 + (n>=2) - if(n%2, num_zeros(n-1)%2)),
              0,
              "A298952 cumulative, round n/2");
}

\\-----------------------------------------------------------------------------
\\ A242179 T(0,0) = 1, T(n+1,2*k) = - T(n,k), T(n+1,2*k+1) = T(n,k), k=0..n, triangle read by rows.
\\
\\ 1,
\\ -1, 1,
\\ 1, -1, -1, 1,
\\ -1, 1, 1, -1,  1, -1, -1, 1,
\\ 1, -1, -1, 1,  -1,

A242179(n,k) =
{
  n>=0 || error();
  k>=0 || error();
  k<2^n || error();
  (-1)^(n-hammingweight(k));
}
{
  my(want=OEIS_samples("A242179"),
     got=[]);
  for(n=0,#want,
     if(#got >= #want, break);
     got=concat(got, vector(2^n,k,k--; A242179(n,k))));
  got=Vec(got,#want);
  check_equal(got, want,
              "A242179(n,k) vs samples, num zeros triangle rows");

  \\ definition formulas
  my(T=A242179);
  check_equal(vector(8,n,n--; vector(2^n,k,k--; T(n+1,2*k))),
              vector(8,n,n--; vector(2^n,k,k--; - T(n,k))));
  check_equal(vector(8,n,n--; vector(2^n,k,k--; T(n+1,2*k+1))),
              vector(8,n,n--; vector(2^n,k,k--; T(n,k))));
}
A242179_flat(n) =
{
  n>=0 || error();
  n++; -(-1)^(logint(n,2)-hammingweight(n));
}
{
  my(want=OEIS_samples("A242179")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A242179_flat(n)),
              want,
              "A242179_flat() vs samples");

  check_equal(vector(1024,n,n--; my(f=A242179_flat(n));
                                 if(f==1,1, f==-1,0, error())),
              vector(1024,n,     if(n==0,0, A298952(n))),
              "A242179 flat vs A298952 1,0");

  check_equal(vector(1024,n,n--; my(f=A242179_flat(n));
                                 if(f==1,0, f==-1,1, error())),
              vector(1024,n,     A059448(n)),
              "A242179 flat vs A059448 0,1");
}

\\ select(n->A242179_flat(n)==1, [2..30])
\\ select(n->A242179_flat(n)==-1, [2..30])
\\ not in OEIS: 2, 3, 6, 8, 9, 11, 14, 15, 18, 20, 21, 24, 25, 27, 30
\\ not in OEIS: 4, 5, 7, 10, 12, 13, 16, 17, 19, 22, 23, 26, 28, 29


\\-----------------------------------------------------------------------------
check_equal(n,'n);
check_equal(C,'C);
check_equal(p,'p);
check_equal(k,'k);
check_equal(i,'i);
print("end");
