\\ Copyright 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

arg_0_to_2pi(z) = my(a=arg(z)); if(a<0,a+=2*Pi); a;
lex_by_abs(a,b) = \
  if(abs(a) == abs(b), lex(arg_0_to_2pi(a),arg_0_to_2pi(b)), lex(abs(a),abs(b)));
lex_by_abs_descending(a,b) = - lex_by_abs(a,b);


\\-----------------------------------------------------------------------------

n_to_other(n) =
{
  my(z = dragon_point_by_sum(n),
     an1 = dragon_unpoint(z,1),
     an2 = dragon_unpoint(z,-1));
  if(! ((an1[1] == 0 && an1[2] == n)
        || (an2[1] == 0 && an2[2] == n)),
     error("oops, ",an1," ",an2));
  if(an1[1] == 0 && an1[2] != n,  an1[2],
     an2[1] == 0 && an2[2] != n,  an2[2],
     -1);
}

binary_to_b_powers(v) =
{
  my(s=1);
  for(i=2,#v, v[i]*=s; if(v[i]&&!v[i-1], s = -s));
  v;
}
binary_to_b_powers_str(v) = \
  concat(apply(x->Strprintf("%2d ",x),binary_to_b_powers(v)));

\\-----------------------------------------------------------------------------

{
  state_transition = [0,1, 1,2;    \\ 1  find low 1
                      0,6, 1,3;    \\ 2  low 1, now second bit 01 or 11
                      1,10, 0,3;    \\ 3  flip until 0, for 11
                      0,4, 1,4;    \\ 4  unchanging
                      0,6, 1,3;    \\ 5  this unchanged, then flip again
                      1,9, 0,7;    \\ 6  low 01, now three 001 or 101
                      1,8, 0,7;    \\ 7  flip until 0, for 01
                      0,9, 1,7;    \\ 8  this unchanged, then flip again, for 01
                      1,9, 0,4;    \\ 9  flip until 1
                      0,4, 1,3;    \\ a another 0 to unchanged
                      0,0,0,0];
}


n_to_other_by_bits(n) =
{
  my(old = concat([0,0,0],binary(n)));
  my(new = vector(#old));
  states = vector(#old);
  my(state = 1);
  forstep(i=#old,1,-1,
         states[i] = state;
         my(bit = old[i]);
         new[i] = state_transition[state, 1 + 2*bit];
         state  = state_transition[state, 2 + 2*bit];
         );
  return(from_binary_vector(new));
  


  my(old = concat([0,0,0],binary(n)));
  my(new = old);
  my(flip=1);
  forstep(i=#old,2,-1,
         if(old[i-1]==0 && old[i]==1, flip = 1-flip);
         if(new[i-1]==0 && new[i]==1, flip = 1-flip);
         flip = (new[i]==1 && old[i]==1);
         if(flip,
           for(j=1,i-2,new[j]=1-new[j]))
         );
  from_binary_vector(new);
}

show(n) =
{
  print();
  my(z = dragon_point_by_sum(n),
     an1 = dragon_unpoint(z,1),
     an2 = dragon_unpoint(z,-1));
  if(an2[2]==n, my(t=an1);an1=an2;an2=t); \\ swap
  my(other = an2[2]);
  if(! (an1[2]==n && an2[2]==other), error("oops ",n,"\n",an1,"\n",an2));
  my(bybits = n_to_other_by_bits(n));
  my(s1=Str(to_binary(n)));
  my(s2=Str(to_binary(other)));
  my(s3=Str(to_binary(bybits)));
  my(s4=concat(apply(x->Strprintf("%x",x),states)));
  my(len=vecmax([length(s1),length(s2),length(s3),length(s4)]));
  printf("%*s   %s %d\n", len,s1, an1, n);
  printf("%*s   %s %d\n", len,s2, an2, other);
  my(diff=if(bybits==other,""," *****"));
  printf("%*s%s\n", len,s3, diff);
  printf("%*s\n", len,s4);

  s1=binary_to_b_powers_str(binary(n));
  s2=binary_to_b_powers_str(binary(other));
  s3=binary_to_b_powers_str(binary(bybits));
  len=vecmax([length(s1),length(s2),length(s3)]);
  printf("[%*s\n", len,s1);
  printf("[%*s\n", len,s2);
  printf("[%*s\n", len,s3);
}


show(from_binary(11110000011111000000001111111011));
show(from_binary(11110000011111000000001111110111111));
show(from_binary(1111100000111111001111111));
show(from_binary(1111000001111101111110111111));
show(from_binary(1111000001111100000111111));
show(from_binary(111110000000011111110000000001));
show(from_binary(1111000001111100000000111111101));

show(from_binary(10101010101010101010101));
show(from_binary(11011011011011011011011011011011011));

