#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.006;
use strict;
use warnings;
use Test::More;

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyFLAT;

use MyTestHelpers;
BEGIN { MyTestHelpers::nowarnings(); }

plan tests => 360;


# Test of fBpred based on contiguous hulls surrounding.
# Bigger than the crossings surround.

#------------------------------------------------------------------------------

is($hullsaround_segs_inner->intersection($hullsaround_segs_outer)->num_segs, 0);
# $hullsaround_segs_outer->print;
# $hullsaround_segs_inner->print;
# $hullsaround_segs_all->print;

{
  my $conf = $hullsaround_segs_all;
  foreach my $bit (0,1) {
    ok($conf->is_subset( $conf->expand($bit) ),
       "$conf->{'name'}, bit=$bit");
  }
}

my (@hullsaround_state_to_conf,
    @fBpred_hullsaround_state_table,
    $fNonRpred_hullsaround_flat,$fNonLpred_hullsaround_flat,
    $fNonBpred_hullsaround_flat,
    $fRpred_hullsaround_flat,$fLpred_hullsaround_flat,
    $fBpred_hullsaround_flat);
{
  my $verbose = 0;
  my @state_enclosed_lists;
  my @hullsaround_state_is_RLB_enclosed;

  {
    my %conf_to_state;
    sub conf_to_state {
      my ($conf) = @_;
      my $key = $conf->key;
      my $state = $conf_to_state{$key};
      if (! defined $state) {
        push @hullsaround_state_to_conf, $conf;
        $state = $#hullsaround_state_to_conf;
        $conf_to_state{$key} = $state;
        # print "new state $state\n";
      }
      return $state;
    }
  }
  conf_to_state(MyDragonConf->initial(within => $hullsaround_segs_all));

  {
    my $state = 0;
    my $depth = 0;
    for (;;) {
      $depth++;
      my $last_state = $#hullsaround_state_to_conf;
      for ( ; $state <= $last_state; $state++) {
        my $from_conf = $hullsaround_state_to_conf[$state];
        if ($verbose >= 3) {
          print "########################################\n";
          print "from $state  ",$from_conf->key,"\n";
          $from_conf->print;
        }
        foreach my $bit (0,1) {
          my $to_conf = $from_conf->expand($bit);
          my $to_state = conf_to_state($to_conf);
          $fBpred_hullsaround_state_table[$state]->[$bit] = $to_state;

          if ($verbose >= 3) {
            print "  bit=$bit to $fBpred_hullsaround_state_table[$state]->[$bit]  ",$to_conf->key,"\n";
            $to_conf->print;
          }
        }
      }
      my $new = $#hullsaround_state_to_conf - $last_state;
      if ($verbose) {
        print "depth $depth new $new\n";
      }
      last if $new == 0;
    }
  }

  my $num_states = scalar(@fBpred_hullsaround_state_table);
  diag("hullsaround fBpred_hullsaround_state_table length $num_states");

  foreach my $i (0 .. $#hullsaround_state_to_conf) {
    my $conf = $hullsaround_state_to_conf[$i];
    foreach my $side (0, 1) {
      $hullsaround_state_is_RLB_enclosed[$i]->[$side]
        = ! $conf->is_boundary($side);
    }
    $hullsaround_state_is_RLB_enclosed[$i]->[2]
      = $hullsaround_state_is_RLB_enclosed[$i]->[0]
      && $hullsaround_state_is_RLB_enclosed[$i]->[1];
  }
  foreach my $side (0, 1, 2) {
    $state_enclosed_lists[$side]
      = [ grep {$hullsaround_state_is_RLB_enclosed[$_]->[$side]} 0 .. $#fBpred_hullsaround_state_table ];
  }

  if ($verbose) {
    print "  enclosed R = ",join(',',@{$state_enclosed_lists[0]}),
      " count ",scalar(@{$state_enclosed_lists[0]}),"\n";
    print "  enclosed L = ",join(',',@{$state_enclosed_lists[1]}),
      " count ",scalar(@{$state_enclosed_lists[1]}),"\n";
    print "  enclosed B = ",join(',',@{$state_enclosed_lists[2]}),
      " count ",scalar(@{$state_enclosed_lists[2]}),"\n";
  }

  foreach my $side (2, 1, 0) {
    foreach my $i (@{$state_enclosed_lists[$side]}) {
      foreach my $bit (0,1) {
        my $to = $fBpred_hullsaround_state_table[$i]->[$bit];
        unless ($hullsaround_state_is_RLB_enclosed[$to]->[$side]) {
          $hullsaround_state_to_conf[$i]->print;
          $hullsaround_state_to_conf[$i]->expand($bit)->print;
          print "oops, side=$side enclosed $i bit=$bit didn't remain so";
          # print $conf->is_boundary($side);
          exit 1;
        }
      }
    }
  }

  {
    # R
    my $f = MyFLAT::aref_to_FLAT_DFA(\@fBpred_hullsaround_state_table,
                                     name => 'fNonRpred_hullsaround',
                                     accepting_list => $state_enclosed_lists[0],
                                    );
    $f = MyFLAT::eventually_accepting($f);
    # $f = $f->MyFLAT::minimize;
    $fNonRpred_hullsaround_flat = $f;
    if ($verbose >= 2) {
      print "$f->{name} accepting ",join(' ',$f->get_accepting),"\n";
    }

    $f = $f->complement;
    $f->{'name'} = 'fRpred_hullsaround';
    $fRpred_hullsaround_flat = $f;
  }
  {
    # L
    my $f = MyFLAT::aref_to_FLAT_DFA(\@fBpred_hullsaround_state_table,
                                     name => 'fNonLpred_hullsaround',
                                     starting => 0,
                                     accepting_list => $state_enclosed_lists[1],
                                    );
    $f = MyFLAT::eventually_accepting($f);
    # $f = $f->MyFLAT::minimize;
    $fNonLpred_hullsaround_flat = $f;

    $f = $f->complement;
    $f->{'name'} = 'fLpred_hullsaround';
    $fLpred_hullsaround_flat = $f;
  }
  {
    # B
    my $f = MyFLAT::aref_to_FLAT_DFA(\@fBpred_hullsaround_state_table,
                                     name => 'fNonBpred_hullsaround',
                                     starting => 0,
                                     accepting_list => $state_enclosed_lists[2],
                                    );
    $f = MyFLAT::eventually_accepting($f);
    # $f = $f->MyFLAT::minimize;
    $fNonBpred_hullsaround_flat = $f;
    if ($verbose >= 2) {
      print "$f->{name} accepting ",join(' ',$f->get_accepting),"\n";
    }
    $f = $f->complement;
    $f->{'name'} = 'fBpred_hullsaround';
    $fBpred_hullsaround_flat = $f;
  }

}

ok($fNonRpred_hullsaround_flat->equals(fNonRpred_flat()),
   "fNonRpred_hullsaround_flat = fNonRpred_flat()");
ok($fNonLpred_hullsaround_flat->equals(fNonLpred_flat()));
ok($fNonBpred_hullsaround_flat->equals(fNonBpred_flat()));

ok($fRpred_hullsaround_flat->equals(fRpred_flat()));
ok($fLpred_hullsaround_flat->equals(fLpred_flat()));
ok($fBpred_hullsaround_flat->equals(fBpred_flat()));

#------------------------------------------------------------------------------
# fBpred hullsaround which are fully enclosed or eventually fully enclosed
#  =
# those with all of $triple_15ths_union_segs

ok ($hullsaround_segs_inner->is_subset($surround_segs_all));

foreach my $state (0 .. $#fBpred_hullsaround_state_table) {
  my $conf = $hullsaround_state_to_conf[$state];
  my $by_table = $fNonBpred_hullsaround_flat->is_accepting($state) ? 1 : 0;
  my $by_segs  = $hullsaround_segs_inner->is_subset($conf) ? 1 : 0;

  if ($by_table != $by_segs) {
    $triple_15ths_union_segs->print;
    $conf->print;
    $triple_15ths_union_segs->difference($conf)->print;
  }
  is ($by_segs, $by_table);
}


#------------------------------------------------------------------------------
exit 0;
