#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90','xy_rotate_minus90','transform_to_horizontal',
  'BlobN','B';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


print "  % generated by tools/$FindBin::Script\n";
# prefer $level even so segments are horizontal and vertical
my $level = 18;
if (@ARGV) { $level = shift @ARGV; }

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my $scale = 2**($level/2);
print "  % level $level\n";
print "  \\begin{scope}[scale=1/$scale]\n";

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

sub xyxy_is_straight_or_diagonal {
  my ($x1,$y1, $x2,$y2) = @_;
  my $dx = $x2 - $x1;
  my $dy = $y2 - $y1;
  return $dx == 0 || $dy == 0 || abs($dx) == abs($dy);
}


# middle of the bridge after blob $k
#                     |    blob 6
#    k            *---N--   even
#  blob 5     |   |        diagonal
#   odd   *---@---*          k+1
# straight    |    
#
sub BridgeMidpoint {
  my ($k) = @_;
  my $n = BlobN($k+1);    # start of blob k+1, end of blob k
  $n -= ($k % 2 ? 2 : 1); # 
  return $path->n_to_xy($n);
}

sub BridgeMidpoint_at_end {
  my ($k) = @_;
  my ($x,$y) = BridgeMidpoint($k);
  ($x,$y) = xy_rotate_minus90($x,$y);
  return ($x_hi + $x, $y_hi + $y);
}

{
  $Printwrap = 2;
  printwrap("\\draw[my line] (0,0)");
  $Printwrap += 2;
  my ($prev_x,$prev_y);
  foreach my $k (4 .. $level) {
    my ($x,$y) = BridgeMidpoint($k);
    ($x,$y) = transform($x,$y);
    printwrap("--($x,$y)");
    ### bridge: "$x, $y"

    if (defined $prev_x) {
      xyxy_is_straight_or_diagonal($prev_x,$prev_y, $x,$y)
        or die "Oops, not straight or diagonal";
    }
    ($prev_x,$prev_y) = ($x,$y);
  }
  printwrap_newline();

  ### ends
  undef $prev_x;
  foreach my $k (reverse 4 .. $level-3) {
    my ($x,$y) = BridgeMidpoint_at_end($k);

    ($x,$y) = transform($x,$y);
    printwrap("--($x,$y)");
    ### bridge: "$x, $y"

    if (defined $prev_x) {
      xyxy_is_straight_or_diagonal($prev_x,$prev_y, $x,$y)
        or die "Oops, not straight or diagonal end k=$k $prev_x,$prev_y to $x,$y";
    }
    ($prev_x,$prev_y) = ($x,$y);
  }
  {
    my ($x,$y) = transform($x_hi,$y_hi);
    printwrap("--($x,$y);\n");
  }
}

print "  \\end{scope}\n";


print "  % end generated\n";
exit 0;
