#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'max';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'chamfer','xy_rotate_plus90','xy_rotate_minus90','RQ';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

print "  \\node (start) at (0,0) [my dot];\n";
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  print "  \\coordinate (end) at ($x,$y) [my dot];\n";
}

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $segments->add($x1,$y1, $x2,$y2);
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

print "  \\begin{scope}[font=\\scriptsize]\n";
my @RQsides;
for (my $n = 0; $n < $n_hi; ) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
  my $cx = $x + $dx/2 + $rx/2;
  my $cy = $y + $dy/2 + $ry/2;

  ($dx,$dy) = xy_rotate_minus90($dx,$dy);
  my $xs = $x . chamfer($dx,'\\MyGap');
  my $ys = $y . chamfer($dy,'\\MyGap');
  $Printwrap = 4;
  printwrap(" \\draw[dotted] ($xs,$ys)");
  $Printwrap += 2;
  my $s = 3;
  for (;;) {
    $x += $dx;
    $y += $dy;

    if (my @n_list = $path->xy_to_n_list($x,$y)) {
      $n = max(@n_list);
      last;
    }
    printwrap("--($x,$y)");
    ($dx,$dy) = xy_rotate_plus90($dx,$dy);
    $s--;
  }
  $s >= 0 or die;

  $xs = $x . chamfer(-$dx,'\\MyGap');
  $ys = $y . chamfer(-$dy,'\\MyGap');
  printwrap("--($xs,$ys);");

  $Printwrap = 4;
  printwrap(" \\node at ($cx,$cy) {$s};\n");
  push @RQsides, $s;
}

scalar(@RQsides) == RQ($level) or die;
print "  \\end{scope}\n";

print "% GP-Test  RQ($level) == ",scalar(@RQsides),"\n";
print "% GP-Test  vector(",scalar(@RQsides),",m,m--; RQsides(m)) == [",
  join(',',@RQsides),"]\n";
print "  % end generated\n";
exit 0;
