#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_minus90','xy_rotate_plus90','mul',
  'transform_to_horizontal','BlobN','Rnext','Lnext_k';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 16;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
print "  % A high-resolution approximation to the middle biggest blob,\n";
print "  % positioned for whole curve (0,0) to (1,0).  Can fill or draw.\n";
print "  % Lines are drawn connecting the middles of boundary squares.\n";
print "  my blob fractal pic/.pic={\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $scale = int(($level+2)/2);
print "  \\begin{scope}[scale=1/2^$scale]\n";

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level+1);  # +1 for mul()
}

my $segments = MyLineSegments->new;

# $side==1 for right, $side==-1 for left
sub n_to_cxcy {
  my ($n, $side) = @_;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my $dx = $x2 - $x1;
  my $dy = $y2 - $y1;

  my $cx = ($x1+$x2 + $side*$dy)/2;
  my $cy = ($y1+$y2 - $side*$dx)/2;
  return mul($cx,$cy);
}

my $n_start = BlobN($level) - 1;
my $n_end = BlobN($level+1) - 3;
### $n_start
### $n_end

my ($x_start,$y_start) = mul($path->n_to_xy($n_start));
my ($x_end,$y_end)     = mul($path->n_to_xy($n_end + 1));

my $BlobRQ = 0;
{
  my ($prev_cx,$prev_cy) = ($x_start,$y_start);
  for (my $n = $n_start; $n <= $n_end; $n = Rnext($n)) {
    my ($cx,$cy) = n_to_cxcy($n,1);
    ### $n
    ### cxcy: "$cx,$cy"

    $BlobRQ += ($cx != $prev_cx || $cy != $prev_cy);
    $segments->add($prev_cx,$prev_cy, $cx,$cy);
    $prev_cx = $cx;
    $prev_cy = $cy;
  }
  $segments->add($prev_cx,$prev_cy, $x_end,$y_end);
}
my $BlobLQ = 0;
{
  my ($prev_cx,$prev_cy) = ($x_start,$y_start);
  for (my $n = $n_start; $n <= $n_end; $n = Lnext_k($level,$n)) {
    defined $n || die;
    my ($cx,$cy) = n_to_cxcy($n,-1);
    ### cxcy: "$cx,$cy"

    $BlobLQ += ($cx != $prev_cx || $cy != $prev_cy);
    $segments->add($prev_cx,$prev_cy, $cx,$cy);
    $prev_cx = $cx;
    $prev_cy = $cy;
  }
  $segments->add($prev_cx,$prev_cy, $x_end,$y_end);
}

$Printwrap = 2;
printwrap("\\path[pic actions] ");
$Printwrap += 2;
$segments->printwrap (transform => \&transform);
printwrap(";\n");
my $draw_count = $segments->lines_joined_count_points;


print "  \\end{scope}\n";
print "% GP-Test-Last  BlobRQ($level) == $BlobRQ\n";
print "% GP-Test-Last  BlobLQ($level) == $BlobLQ\n";
print "  % drawn with $draw_count points\n";
print "  },\n";
print "  % end generated\n";
exit 0;
