#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::ComplexMinus 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon '@dir4_to_dx','@dir4_to_dy';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::ComplexMinus->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $insides = MyLineSegments->new;
my $boundary = MyLineSegments->new;

print "  \\begin{scope}[font=\\tiny]\n";

# numbers
{
  $Printwrap = 4;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    $x = 2*$x;
    $y = 2*$y - 1;
    printwrap(" \\node at ($x,$y) {$n};");
  }
  printwrap_newline();
}

{
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    foreach my $dir4 (0 .. 3) {
      my $dx = $dir4_to_dx[$dir4];
      my $dy = $dir4_to_dy[$dir4];
      my $n_other = $path->xy_to_n($x+$dx,$y+$dy);
      my $segs = (defined $n_other && $n_other <= $n_hi
                  ? $insides     # inside side
                  : $boundary);  # boundary side
      $segs->add(2*$x+$dx - $dy,  2*$y+$dy - $dx - 1,
                 2*$x+$dx + $dy,  2*$y+$dy + $dx - 1);
    }
  }

  # outer boundary
  {
    $Printwrap = 4;
    printwrap("\\draw ");
    $Printwrap += 2;
    $boundary->printwrap;
    printwrap(";\n");
  }

  # inner lines, dotted
  {
    $Printwrap = 4;
    printwrap("\\draw[dotted] ");
    $Printwrap += 2;
    $insides->printwrap;
    printwrap(";\n");
  }

  # inside corners
  {
    my $is_visited = sub {
      my ($x,$y) = @_;
      my $n = $path->xy_to_n($x,$y);
      return (defined $n && $n <= $n_hi);
    };
    $Printwrap = 4;
    printwrap("\\path foreach \\p in {");
    $Printwrap = 6;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      if ($is_visited->($x+1,$y)
          && $is_visited->($x+1,$y+1)
          && $is_visited->($x,$y+1)) {
        $x = 2*$x + 1;
        $y = 2*$y;
        printwrap("($x,$y),");
      }
    }
    printwrap("}");
    printwrap(" { \\p node[my dot] };\n");
  }
}

printwrap_newline();
print "  \\end{scope}\n";

# /2 because 2*X,2*Y coordinates
my $boundary_length = $boundary->length / 2;
my $insides_length = $insides->length / 2;
print "  % boundary length $boundary_length insides $insides_length\n";
print "% GP-Test  TS($level) == $boundary_length\n";
print "% GP-Test  TD($level) == $insides_length\n";

print "  % end generated\n";
exit 0;
