#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'max','sum';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'hull_expand';

use Math::PlanePath::GcdRationals;
BEGIN { *gcd = \&Math::PlanePath::GcdRationals::_gcd; }

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 20;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n";

# sub xyxy_num_integer_points {
#   my ($x1,$y1, $x2,$y2) = @_;
#   my $dx = $x2-$x1;
#   my $dy = $y2-$y1;
#   return gcd($dx,$dy);
# }
# CHECK {
#   xyxy_num_integer_points(0,0, 2,0) == 2 or die;
#   xyxy_num_integer_points(0,0, 2,1) == 1 or die;
#   xyxy_num_integer_points(0,0, 4,2) == 2 or die;
#   xyxy_num_integer_points(0,0, 6,3) == 3 or die;
# }


my $points = [ [0,0], [1,0] ];
my $ex = 1;
my $ey = 0;
my $path = Math::PlanePath::DragonCurve->new;

my @Pv;
my @Pgaps;
my @HBP;
my @HBV;
my @HBVgaps;
my @HBS;
foreach my $k (0 .. $max_level) {
  my ($n_lo,$n_hi) = $path->level_to_n_range($k);

  my $HBP = 0;
  my $HBS = 0;
  my @Pv_row;
  my @Pgaps_row;
  foreach my $i (0 .. $#$points) {
    my $p1 = $points->[$i];
    my $p2 = $points->[($i+1) % scalar(@$points)];
    my ($x1,$y1) = @$p1;
    my ($x2,$y2) = @$p2;
    ### side: "$x1,$y1 to $x2,$y2"

    my $gaps = 0;
    my $V_side = 0;
    my $dx = $x2 - $x1;
    my $dy = $y2 - $y1;
    my $g = gcd($dx,$dy);
    $dx /= $g;
    $dy /= $g;
    my $step_by_1 = (abs($dx) <= 1 && abs($dy) <= 1);
    my $step_by_all = (abs($dx) == abs($x1-$x2) && abs($dy) == abs($y1-$y2));
    $step_by_1 || $step_by_all || die;

    my $prev_V_visited = 1;
    while ($x1 != $x2 || $y1 != $y2) {
      my $n = $path->xy_to_n($x1,$y1);
      my $V_visited = (defined($n) && $n <= $n_hi ? 1 : 0);

      my $ns = $path->xyxy_to_n_either($x1,$y1, $x1+$dx,$y1+$dy);
      my $S_traversed = (defined($ns) && $ns < $n_hi ? 1 : 0);

      ### at: "k=$k P".($i+1)." $x1,$y1 is v=$v"
      $V_side += $V_visited;
      $HBS += $S_traversed;
      $gaps += ($V_visited && ! $prev_V_visited);
      $HBP++;

      $prev_V_visited = $V_visited;
      $x1 += $dx;
      $y1 += $dy;
    }
    $gaps += ! $prev_V_visited;  # final point is a v=1
    $gaps += ! $step_by_1;  # sides P5 and P9 are gaps
    push @Pv_row, $V_side;
    push @Pgaps_row, $gaps;
  }
  if ($k==0) { $HBS = 1; }
  ### @Pv_row
  push @Pv, \@Pv_row;
  push @Pgaps, \@Pgaps_row;
  push @HBP, $HBP;
  push @HBV, sum(@Pv_row);
  push @HBVgaps, sum(@Pgaps_row);
  push @HBS, $HBS;
  ($points,$ex,$ey) = hull_expand($points,$ex,$ey);
}
### @Pv

print "% GP-DEFINE  Pv_generated_samples = {[\n";
foreach my $k (0 .. $#Pv) {
  my $sep = ($k == $#Pv ? '' : ',');
  print "% GP-DEFINE    [",join(',',@{$Pv[$k]}),"]$sep  /* k=$k */\n";
}
print "% GP-DEFINE  ]};\n";

print "% GP-DEFINE  Pgaps_generated_samples = {[\n";
foreach my $k (0 .. $#Pgaps) {
  my $sep = ($k == $#Pgaps ? '' : ',');
  print "% GP-DEFINE    [",join(',',@{$Pgaps[$k]}),"]$sep  /* k=$k */\n";
}
print "% GP-DEFINE  ]};\n";

print "% GP-DEFINE  HBP_generated_samples = [",join(',',@HBP),"];\n";
print "% GP-DEFINE  HBV_generated_samples = [",join(',',@HBV),"];\n";
print "% GP-DEFINE  HBVgaps_generated_samples = [",join(',',@HBVgaps),"];\n";
print "% GP-DEFINE  HBS_generated_samples = [",join(',',@HBS),"];\n";

print "% end generated\n";
exit 0;
