#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Lingua::EN::Numbers;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'hull_expand','xy_to_tex_str';
use MyLineSegments;
use MyPrintwrap ':all';


$Printwrap = 2;
printwrap("% generated by tools/$FindBin::Script\n");

my $level = 6;

{
  my $path = Math::PlanePath::DragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi-1) {
    $segments->add($path->n_to_xy($n),
                   $path->n_to_xy($n + 1));
  }
  $Printwrap = 2;
  printwrap("\\draw[my grey] ");
  $Printwrap = 4;
  $segments->printwrap;
  printwrap(";\n");
}

my $ex = 1;
my $ey = 0;
my $points = [ [0,0],
               [$ex,$ey],
             ];
foreach (1 .. $level) {
  ($points,$ex,$ey) = hull_expand($points,$ex,$ey);
}

{
  $Printwrap = 2;
  printwrap("\\draw\n");
  $Printwrap = 4;
  my $i = 1;
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    printwrap("($x,$y) node[my dot,name=P$i] --\n");
    $i++;
  }
  printwrap("cycle;\n");
}

{
  my $i = 1;
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    my $name = Lingua::EN::Numbers::num2en($i);
    my $str = xy_to_tex_str($x,$y);
    print "  \\newcommand{\\MyP$name}{$str}\n";
    $i++;
  }
}
{
  my $i = 1;
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    print "% GP-Test   P$i($level) == ($x)+I*$y\n";
    $i++;
  }
}

print "  % end generated\n";
exit 0;
