#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_minus90','xy_rotate_plus90',
  'transform_to_horizontal','BlobN','Rnext','Lnext_k';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


# must odd level so lines vertical/horizontal
my $level = 15;
if (@ARGV) { $level = shift @ARGV; }

$level % 2 == 1 or die "level must be odd";
print "  % generated by tools/$FindBin::Script $level\n";
print "  % A high-resolution approximation to the dragon curve,\n";
print "  % going from (0,0) to (1,0).  Can fill or draw.\n";
print "  % Lines are drawn connecting the middles of boundary squares.\n";
print "  my dragon fractal pic/.pic={\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
my $scale = ($level+1)/2;
my $extra_scale = 0;
if ($scale > 8) {
  $extra_scale = $scale - 8;
  $scale = 8;
}
print "  \\begin{scope}[scale=1/2^$scale]\n";

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

# $side==1 for right, $side==-1 for left
sub n_to_cxcy {
  my ($n, $side) = @_;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my $dx = $x2 - $x1;
  my $dy = $y2 - $y1;

  my $cx = ($x1+$x2 + $side*$dy)/2;
  my $cy = ($y1+$y2 - $side*$dx)/2;
  return transform($cx,$cy);
}

my $segments = MyLineSegments->new;

my $RQ = 0;
{
  my ($prev_cx,$prev_cy) = (0,0);
  for (my $n = 0; $n < $n_hi; $n = Rnext($n)) {
    my ($cx,$cy) = n_to_cxcy($n,1);
    ### cxcy: "$cx,$cy"

    $RQ += ($cx != $prev_cx || $cy != $prev_cy);
    $segments->add($prev_cx,$prev_cy, $cx,$cy);
    $prev_cx = $cx;
    $prev_cy = $cy;
  }
  $segments->add($prev_cx,$prev_cy, transform($x_hi,$y_hi));
}
my $LQ = 0;
{
  my ($prev_cx,$prev_cy) = (0,0);
  for (my $n = 0; defined $n; $n = Lnext_k($level,$n)) {
    defined $n || die;
    my ($cx,$cy) = n_to_cxcy($n,-1);
    ### cxcy: "$cx,$cy"

    $LQ += ($cx != $prev_cx || $cy != $prev_cy);
    $segments->add($prev_cx,$prev_cy, $cx,$cy);
    $prev_cx = $cx;
    $prev_cy = $cy;
  }
  $segments->add($prev_cx,$prev_cy, transform($x_hi,$y_hi));
}

$Printwrap = 2;
printwrap("\\path[pic actions] ");
$Printwrap += 2;
my $extra_scale_factor = 1/2**$extra_scale;
$segments->printwrap (transform => sub {
                        my ($x,$y) = @_;
                        return ($x*$extra_scale_factor, $y*$extra_scale_factor);
                      });
printwrap(";\n");
my $draw_count = $segments->lines_joined_count_points;

print "  \\end{scope}\n";
print "% GP-Test-Last  RQ($level) == $RQ\n";
print "% GP-Test-Last  LQ($level) == $LQ\n";
print "  % drawn with $draw_count points\n";
print "  },\n";
print "  % end generated\n";
exit 0;









__END__

# Previous code drawing boundary segments, including hanging squares etc.

# # prefer even level so lines vertical/horizontal
# my $level = 14;
# if (@ARGV) { $level = shift @ARGV; }
#
# $level % 2 == 0 or die "level must be even";
# print "% generated by tools/$FindBin::Script\n";
# print "\\tikzset{\n";
# print "  my dragon fractal pic/.pic={\n";
#
# my $path = Math::PlanePath::DragonCurve->new;
# my ($n_lo, $n_hi) = $path->level_to_n_range($level);
# my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
# my $scale = sqrt($x_hi**2 + $y_hi**2);
# print "  \\begin{scope}[scale=1/$scale]\n";
#
# sub transform {
#   my ($x,$y) = @_;
#   return transform_to_horizontal($x,$y, $level);
# }
#
# sub path_n_is_right_boundary {
#   my ($path, $n, $n_limit) = @_;
#   ### path_n_is_right_boundary(): $n
#   my ($x1,$y1) = $path->n_to_xy($n);
#   my ($x2,$y2) = $path->n_to_xy($n+1);
#   my ($dx,$dy) = $path->n_to_dxdy($n);
#   ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
#   my $x1s = $x1 + $dx;
#   my $y1s = $y1 + $dy;
#   my $x2s = $x2 + $dx;
#   my $y2s = $y2 + $dy;
#   return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
#           || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
#           || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
# }
# sub path_n_is_left_boundary {
#   my ($path, $n, $n_limit) = @_;
#   ### path_n_is_left_boundary(): $n
#   my ($x1,$y1) = $path->n_to_xy($n);
#   my ($x2,$y2) = $path->n_to_xy($n+1);
#   my ($dx,$dy) = $path->n_to_dxdy($n);
#   ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to side
#   my $x1s = $x1 + $dx;
#   my $y1s = $y1 + $dy;
#   my $x2s = $x2 + $dx;
#   my $y2s = $y2 + $dy;
#   return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
#           || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
#           || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
# }
# sub path_xyxy_is_traversed_either {
#   my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
#   my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
#   return defined $n && $n < $n_limit;
# }
#
# my %blobs;
# foreach my $k (4 .. $level) {
#   my $n = BlobN($k);
#   $blobs{$n} = 'draw';   # blob start
#   $n = BlobN($k+1) - 3;
#   $blobs{$n} = 'pic actions';   # blob end
# }
# foreach my $k (4 .. $level-2) {
#   my $n = BlobN($k);
#   $blobs{$n_hi-$n} = 'pic actions';  # blob start from end, is blob end from start
#   $n = BlobN($k+1) - 3;
#   $blobs{$n_hi-$n} = 'draw';
# }
# {
#   my $n = BlobN($level+1) - 3;
#   $blobs{$n} = 'pic actions,#1';   # blob end of middle biggest
# }
# ### %blobs
#
# $Printwrap = 2;
# my $count_segments = 0;
# my $draw_count = 0;
# my $segments = MyLineSegments->new;
# my $segments_middle_biggest;
#
# sub segments_flush {
#   my ($type) = @_;
#   if ($type eq 'middle') {
#     print "  \\pic at (0,0) [pic actions] {my blob fractal unscaled pic={#1}};\n";
#     $segments_middle_biggest = $segments;
#   } else {
#     $Printwrap = 2;
#     if ($type eq 'draw') {
#       printwrap("\\draw ");
#     } else {
#       printwrap("\\path[$type] ");
#     }
#     $Printwrap += 2;
#     $segments->printwrap;
#     printwrap(";\n");
#     $draw_count += $segments->lines_joined_count_points;
#   }
#   $segments = MyLineSegments->new;
# }
#
# foreach my $n ($n_lo .. $n_hi-1) {
#   if (my $type = $blobs{$n}) {
#     segments_flush($type);
#   }
#
#   my $is_left = path_n_is_left_boundary($path,$n, $n_hi);
#   my $is_right = path_n_is_right_boundary($path,$n, $n_hi);
#   next unless $is_left || $is_right;
#
#   my ($x1,$y1) = $path->n_to_xy($n);
#   my ($x2,$y2) = $path->n_to_xy($n+1);
#   ($x1,$y1) = transform($x1,$y1);
#   ($x2,$y2) = transform($x2,$y2);
#   $segments->add($x1,$y1, $x2,$y2);
#   $count_segments++;
#   if ($is_left && $is_right) { $count_segments++; }
# }
#
# segments_flush('draw');
#
# print "  \\end{scope}\n";
#
# print "  % total boundary segments $count_segments drawn with $draw_count points\n";
# print "% GP-Test-Last  B($level) == $count_segments\n";
# print "  },\n";
#
# # an unsuccessful idea to make a pic out of a pic
# if ($segments_middle_biggest) {
#   print "  my blob fractal unscaled pic/.pic={\n";
#   $segments = $segments_middle_biggest;
#   segments_flush('#1');
#   print "  },\n";
#   print "  my blob fractal pic/.pic={\n";
#   print "  \\pic[scale=1/$scale] at (0,0) {my blob fractal unscaled pic={#1}};\n";
#   print "  },\n";
# }
#
# print "}\n";
# print "% end generated\n";
# exit 0;
