#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyDragon '@fVisits_adjacent_list',
  'xy_to_node_name', '@dir4_to_dx','@dir4_to_dy';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

{
  my $conf = MyDragonConf->new
    (name => 'fVisits_adjacent_list',
     side => 0,
     segs => [ grep {defined} @fVisits_adjacent_list ]);

  my $str = $conf->print_to_str;
  $str =~ s/^/  %    /mg;
  print $str;
}

# node dots
my %seen;
foreach my $seg (@fVisits_adjacent_list,
                 '0,0,0') {
  next unless defined $seg;
  my ($x,$y,$dir) = split /,/, $seg;
  foreach (0, 1) {
    my $name = xy_to_node_name($x,$y);
    unless ($seen{$name}++) {
      my $blank = ($x==0 && $y==0) || ($x==1 && $y==0) ? '' : ' blank';
      print "  \\node ($name) at ($x,$y) [my$blank dot];\n";
    }
    $x += $dir4_to_dx[$dir];
    $y += $dir4_to_dy[$dir];
  }
}

# segments
foreach my $s (1 .. $#fVisits_adjacent_list) {
  my $seg = $fVisits_adjacent_list[$s];
  my ($x,$y,$dir) = split /,/, $seg;
  my $from = xy_to_node_name($x,$y);
  my $to   = xy_to_node_name($x+$dir4_to_dx[$dir], $y+$dir4_to_dy[$dir]);
  print "  \\draw[->] ($from)--($to);\n";
}
foreach my $s (1 .. $#fVisits_adjacent_list) {
  my $seg = $fVisits_adjacent_list[$s];
  my ($x,$y,$dir) = split /,/, $seg;
  my $x2 = $x + $dir4_to_dx[$dir];
  my $y2 = $y + $dir4_to_dy[$dir];
  print "  \\coordinate (s$s-start) at ($x,$y); \\coordinate (s$s-end) at ($x2,$y2);\n";
}

print "  % end generated\n";
exit 0;
