#!/usr/bin/perl -w

# Copyright 2019, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'ffixed_u','ffixed_du', 'ffixed_u_parent', 'ffixed_row_w';
use MyPrintwrap ':all';

my $max_row = 6;
my $type = 'u';
if (@ARGV) { $type = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $type\n";

my %type_to_func = (m => \&noop,
                    u => \&ffixed_u,
                    du => \&ffixed_du);
my $func = $type_to_func{$type} or die;

sub noop { $_[0] }

my $m_start   = ($type eq 'm' ? 1 : 0);
my $row_start = ($type eq 'm' ? 1 : 0);
$Printwrap = 2;
my %nodes;
foreach my $row ($row_start .. $max_row) {
  my $max_m = ($row==0 ? 0 : ffixed_row_w($row-1));
  foreach my $m ($m_start .. $max_m) {
    my $value = $func->($m);
    my $node = "row$row-$m";
    printwrap(" \\node ($node) at ($node) [my box] {\$$value\$};");
    $nodes{$node} = 1;
  }
  printwrap_newline();

  if ($row >= 1) {
    my $prev_row = $row-1;
    foreach my $m ($m_start .. $max_m) {
      last if $row == 1 && $m == 1;
      my $parent = ffixed_u_parent($m);
      my $parent_node = "row$prev_row-$parent";
      exists $nodes{$parent_node} or die "oops, no parent $parent_node";
      my $style = ($type eq 'du' && $m%3 == 1 ? 'my dotted' : '->');
      printwrap(" \\draw[$style] ($parent_node) to (row$row-$m);");
    }
    printwrap_newline();
  }
}

print "% GP-Test-Last  vector(",$max_row+1,",k,k--; ffixed_row_w(k)) == \\\n";
print "% GP-Test-Last    [",join(',',map {ffixed_row_w($_)} 0 .. $max_row),"]\n";

my $max_w = ffixed_row_w($max_row);
print "% GP-Test-Last  vector(",$max_w,",m,m--; ffixed_$type(m)) == \\\n";
print "% GP-Test-Last    [",join(',',map {$func->($_)} 0 .. $max_w-1),"]\n";
print "% GP-Test-Last  vector(",$max_w,",m,m--; ffixed_u_parent(m)) == \\\n";
print "% GP-Test-Last    [",join(',',map {ffixed_u_parent($_)} 0 .. $max_w-1),"]\n";

print "  % end generated\n";
exit 0;
