#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()
use Math::PlanePath::DragonMidpoint 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 2;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_lo,$y_lo) = $path->n_to_xy($n_lo);
print "  \\node (start) at ($x_lo,$y_lo) [my dot];\n";
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
print "  \\node (end) at ($x_hi,$y_hi) [my dot];\n";

$path = Math::PlanePath::DragonMidpoint->new;
($n_lo, $n_hi) = $path->level_to_n_range($level);
my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  $segments->add($x,$y, $x+1,$y);
  $segments->add($x,$y+1, $x+1,$y+1);
  $segments->add($x,$y, $x,$y+1);
  $segments->add($x+1,$y, $x+1,$y+1);
}

sub transform {
  my ($x,$y) = @_;
  ($x,$y) = mul($x,$y);
  $x = ($x+1)/2;
  $y = ($y-1)/2;
  if ($x == $x_lo && $y == $y_lo) { return "start"; }
  if ($x == $x_hi && $y == $y_hi) { return "end"; }
  return "$x,$y";
}

$Printwrap = 2;
printwrap("\\draw");
$Printwrap = 4;
foreach my $line ($segments->lines) {
  my ($x1,$y1, $x2,$y2) = @$line;
  printwrap(" (".transform($x1,$y1).")--(".transform($x2,$y2).")");
}
printwrap(";\n");

my @segs = $segments->segments;
my $count = scalar(@segs);
print "  \\newcommand{\\MyCount}{$count}\n";
print "% GP-Test  MS($level)==$count\n";

print "  % end generated\n";
exit 0;
