#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 6;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

sub xy_to_straight {  # to straight
  my ($x,$y) = @_;
  return div($x,$y);
}
sub xy_to_diagonal {  # to diagonals
  my ($x,$y) = @_;
  ($x,$y) = mul($x,$y);
  return ($x/2,$y/2);
}

my $arms = 4;
my $path = Math::PlanePath::DragonCurve->new (arms => $arms);
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

sub n_to_midpoint {  # with factor *2
  my ($n) = @_;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+$arms);
  return (($x1+$x2),
          ($y1+$y2));
}

{
  # lines

  $Printwrap = 2;
  my $arm0_segments = MyLineSegments->new;
  my $segments = MyLineSegments->new;
  foreach my $arm (0 .. 3) {
    my $n_start = $n_lo + $arm;
    {
      my ($x,$y) = n_to_midpoint($n_start);
      ($x,$y) = xy_to_straight($x,$y);
      ($x,$y) = xy_to_diagonal($x,$y);
      printwrap("\\node at ($x,$y) [my dot];\n");
    }
    my $s = ($arm == 0 ? $arm0_segments : $segments);
    my ($prev_x, $prev_y);
    for (my $n = $n_start; $n <= $n_hi; $n += $arms) {
      my ($x,$y) = n_to_midpoint($n);
      ($x,$y) = xy_to_straight($x,$y);
      if (defined $prev_x) {
        ### add: "$x,$y"
        $s->add($prev_x,$prev_y, $x,$y);
      }
      ($prev_x,$prev_y) = ($x,$y);
    }
  }

  $Printwrap = 2;
  printwrap("\\draw[my thick]");
  $Printwrap += 2;
  $arm0_segments->printwrap (transform => \&xy_to_diagonal);
  printwrap(";\n");

  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap (transform => \&xy_to_diagonal);
  printwrap(";\n");
}

{
  # blocks
  print "  \\begin{scope}[shift=(pos-blocks)]\n";

  my $segments_all = MyLineSegments->new; # outline of all arms
  foreach my $arm (0 .. 3) {
    my $segments_arm = MyLineSegments->new;  # outline of this $arm
    for (my $n = $n_lo + $arm; $n <= $n_hi; $n += 4) {
      my ($x,$y) = n_to_midpoint($n);
      ### block: "$x,$y"

      #        x,y+1
      #
      # x-1,y  x,y   x+1,y
      #
      #        x,y-1
      $segments_arm->xor(xy_to_straight($x+1,$y), xy_to_straight($x,$y+1));
      $segments_arm->xor(xy_to_straight($x,$y+1), xy_to_straight($x-1,$y));
      $segments_arm->xor(xy_to_straight($x-1,$y), xy_to_straight($x,$y-1));
      $segments_arm->xor(xy_to_straight($x,$y-1), xy_to_straight($x+1,$y));
    }

    if ($arm == 0) {
      $Printwrap = 2;
      printwrap("\\fill[my grey]");
      $Printwrap += 2;
      $segments_arm->printwrap (transform => \&xy_to_diagonal);
      printwrap(";\n");
    }
    $segments_all->add_segments($segments_arm);
  }

  $Printwrap = 2;
  printwrap("\\draw");
  $Printwrap += 2;
  $segments_all->printwrap (transform => \&xy_to_diagonal);
  printwrap(";\n");

  print "  \\end{scope}\n";
}

print "  % end generated\n";
exit 0;
