#!/usr/bin/perl -w

# Copyright 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;
use List::Util 'min','max';
use Math::NumSeq::PlanePathTurn;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon '@dir4_to_dx','@dir4_to_dy';
use MyPrintwrap ':all';
use MyLineSegments;

# uncomment this to run the ### lines
# use Smart::Comments;


# my $x_min = -24; my $x_max = 24;
my $x_min = 0; my $x_max = 0;
my $y_min = -10; my $y_max = 10;

print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::DragonCurve->new (arms => 4);
my $seq = Math::NumSeq::PlanePathTurn->new(planepath_object => $path,
                                           turn_type => 'LSR');
my %xy;
my @pending = ([0,0,0]);
my $clip = '';
while (defined(my $p = shift @pending)) {
  ### $p
  my ($x,$y, $colour) = @$p;
  foreach my $dir (0 .. 3) {
    my $dx = $dir4_to_dx[$dir];
    my $dy = $dir4_to_dy[$dir];
    my $x = $x+$dx;
    my $y = $y+$dy;
    unless ($y_min <= $y && $y <= $y_max) {
      # && $x_min <= $x && $x <= $x_max;
      $clip .= "  % clip at $x,$y\n";
      next;
    }

    my $n = $path->xy_to_n($x,$y);
    my $turn = $seq->ith($n) // 0;
    my $arm = $n % 4;
    $n >>= 2;
    if ($arm % 2) { $turn = -$turn; }
    next unless $turn==-1;
    next if $xy{$colour}{$x,$y};

    $xy{$colour}{$x,$y} = 1;
    $x_min = min($x_min,$x);
    $x_max = max($x_max,$x);
    if ($x==1  && $y==0) { $colour = 'east'; }
    if ($x==-1 && $y==0) { $colour = 'west'; }
    if ($x==0 && $y==1)  { $colour = 'north'; }
    if ($x==0 && $y==-1) { $colour = 'south'; }
    push @pending, [$x,$y, $colour];
  }
}

foreach my $colour ('north','south',
                    # ,'east','west'
                   ) {
  $Printwrap = 2;
  printwrap("\\fill[my $colour] foreach \\p in {");
  $Printwrap += 2;
  foreach my $y ($y_min .. $y_max) {
    foreach my $x ($x_min .. $x_max) {
      if ($xy{$colour}{$x,$y}) {
        printwrap("($x,$y),");
      }
    }
  }
  printwrap("}");
  printwrap(" { \\p circle(\\MyRadius) };\n");
}

print $clip;
print "  % end generated\n";
exit 0;
