#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min';
use Math::BigRat;

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90';
use MyPrintwrap ':all';


my $level = 6;
if (@ARGV) {
  $level = shift @ARGV;
}

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $count_lines;

foreach my $small_expansion (1, 3) {
  print "  \\begin{scope}[shift=(pos-$small_expansion)]\n";

  my @points = ([0, 0, 0],
                [Math::BigRat->new(1), 0]);

  foreach my $k (1 .. $level) {
    my @new_points;
    foreach my $i (0 .. $#points-1) {
      my ($x1,$y1, $expansion) = @{$points[$i]};
      my ($x2,$y2)             = @{$points[$i+1]};

      if ($expansion >= $level) {
        push @new_points, [$x1, $y1, $expansion];

      } else {
        my $dx = $x2 - $x1;
        my $dy = $y2 - $y1;
        my $dx4 = $dx/4;
        my $dy4 = $dy/4;
        my ($lx4,$ly4) = xy_rotate_plus90($dx4, $dy4);  # first to left

        push @new_points, [$x1, $y1,
                           $expansion + $small_expansion];
        push @new_points, [$x1 + $dx4 + $lx4,
                           $y1 + $dy4 + $ly4,
                           $expansion + 1];
        push @new_points, [$x2 - $dx4 - $lx4,
                           $y2 - $dy4 - $ly4,
                           $expansion + $small_expansion];
      }
    }

    push @new_points, $points[-1];  # unclosed curve
    @points = @new_points;
  }

  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap = 4;
  my $join = '';
  foreach my $p (@points) {
    my ($x,$y) = @$p;
    printwrap("$join($x,$y)");
    $join = '--';
  }
  printwrap(";\n");

  if ($small_expansion == 3) {
    $count_lines = $#points;
  }
  print "  \\end{scope}\n";
}

print "  \\newcommand{\\MyLines}{$count_lines}\n";
print "  % end generated\n";
exit 0;
