#!/usr/bin/perl -w

# Copyright 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'transform_to_horizontal','mul','div';
use MyLineSegments;
use MyXYConvexHull;
use MyPrintwrap ':all';


# same level as gen-dragon-fractal.pl
#
my $level = 14;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
print "  % level=$level\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

sub transform {
  my ($x,$y) = @_;
  ($x,$y) = transform_to_horizontal($x,$y, $level);
  return ($x, $y);
}

my @points;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  ($x,$y) = transform($x,$y);
  push @points, [$x,$y];
}

{
  my $hull = MyXYConvexHull->new_from_points (@points);
  my $hull_vertices = $hull->vertices;
  my $hull_segments = MyLineSegments->new;
  foreach my $i (0 .. $#$hull_vertices) {
    $hull_segments->add(@{$hull_vertices->[$i]}, @{$hull_vertices->[$i-1]});
  }
  my $XYhullB = $hull_segments->length;

  my $scale = 2**($level/2);
  print "  \\begin{scope}[scale=1/$scale,shift=(pos-0)]\n";
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap = 4;
  $hull_segments->printwrap;
  printwrap(";\n");
  print "  \\end{scope}\n";

  my $num_points = ($level&1 ? $hull->num_points_even : $hull->num_points);
  my $area = $hull->area;
  my $tex_area = $area;
  my $gp_area = $area;
  if ($level&1) {
    if ($area&1) {
      $tex_area = "\tfrac{$area}{2}";
      $gp_area = "$area/2";
    } else {
      $tex_area /= 2;
      $gp_area /= 2;
    }
  }
  # print "% GP-Test-Last  XYpoints($level) == $num_points\n";
  print "% GP-Test  XYhullA($level) == $gp_area\n";
  print "% GP-Test  XYhullB($level) == $XYhullB\n";
}

{
  @points = map {[mul(@$_)]} @points;   # multiply b for rotate +45deg

  my $hull = MyXYConvexHull->new_from_points (@points);
  my $hull_vertices = $hull->vertices;
  my $hull_segments = MyLineSegments->new;
  foreach my $i (0 .. $#$hull_vertices) {
    $hull_segments->add(@{$hull_vertices->[$i]}, @{$hull_vertices->[$i-1]});
  }
  my $DXhullB = $hull_segments->length;

  my $scale = 2**($level/2+1);
  print "  \\begin{scope}[scale=1/$scale,shift=(pos-1)]\n";
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap = 4;
  $hull_segments->printwrap (transform => sub {
                               my ($x,$y) = @_;
                               ($x,$y) = div($x,$y);
                               # $x /= 2;
                               # $y /= 2;
                               return ($x,$y);
                             });
  printwrap(";\n");
  print "  \\end{scope}\n";

  my $num_points = ($level&1 ? $hull->num_points : $hull->num_points_even);
  my $area = $hull->area;
  my $tex_area = $area;
  my $gp_area = $area;
  unless ($level&1) {
    if ($area&1) {
      $tex_area = "\tfrac{$area}{2}";
      $gp_area = "$area/2";
    } else {
      $tex_area /= 2;
      $gp_area /= 2;
    }
  }
  # print "% GP-Test-Last  DXpoints($level) == $num_points\n";
  print "% GP-Test  DXhullA($level) == $gp_area\n";
  print "% GP-Test  DXhullB($level) == $DXhullB/sqrt2\n";
}

print "  % end generated\n";
exit 0;
