#!/usr/bin/perl -w

# Copyright 2014, 2015 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'min';
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $k_limit = 12;
print "% generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::DragonCurve->new;

sub xyxy_is_traversed_either {
  my ($x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined($n) && $n < $n_limit;
}

foreach my $name ('A','AR','AL') {
  print "% GP-DEFINE  G${name}N_samples_by_path = { 1/4 * [\n";

  my @x_total;
  my @y_total;
  foreach my $k (1 .. $k_limit) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);

    my $count = 0;
    my $x_total = 0;
    my $y_total = 0;
    my %seen;
    foreach my $n ($n_lo .. $n_hi - 1) {
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);

      foreach my $side (0,1) {
        my ($dx,$dy) = $path->n_to_dxdy($n);
        if ($side == 0) {
          next if $name eq 'AR';
          ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to left side
        } else {
          next if $name eq 'AL';
          ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
        }
        my $x1s = $x1 + $dx;
        my $y1s = $y1 + $dy;
        my $x2s = $x2 + $dx;
        my $y2s = $y2 + $dy;

        my $xm = min($x1, $x2, $x1s, $x2s);
        my $ym = min($y1, $y2, $y1s, $y2s);
        ### square: "n=$n   $xm,$ym"

        next unless
          xyxy_is_traversed_either($x1,$y1, $x1s,$y1s, $n_hi)
          && xyxy_is_traversed_either($x2,$y2, $x2s,$y2s, $n_hi)
          && xyxy_is_traversed_either($x1s,$y1s, $x2s,$y2s, $n_hi);

        next if $seen{"$xm,$ym"}++;

        $x_total += $x1 + $x2 + $x1s + $x2s;
        $y_total += $y1 + $y2 + $y1s + $y2s;
        $count++;
      }
    }

    {
      my $func = __PACKAGE__->can($name) // die;
      my $a = $func->($k);
      $count == $a or die "k=$k oops count=$count $name=$a";
    }

    my $comma = ($k == $k_limit ? '' : ',');
    print "% GP-DEFINE     ($x_total) + ($y_total)*I$comma   /* k=$k */\n";
    $x_total[$k] = $x_total;
    $y_total[$k] = $y_total;
  }
  print "% GP-DEFINE  ]};\n";
  print "% end generated\n";

  my $AR_start;
  for ($AR_start = 0; AR($AR_start)==0; $AR_start++) {}

  my $AL_start;
  for ($AL_start = 0; AL($AL_start)==0; $AL_start++) {}

  print "  % generated by tools/$FindBin::Script\n";
  foreach my $k ($AR_start .. min($AR_start+4,$#x_total)) {
    my $x_total = $x_total[$k];
    my $y_total = $y_total[$k];
    my $denominator = 4*AR($k);

    $x_total % 2 == 0 or die;
    $y_total % 2 == 0 or die;
    $x_total /= 2;
    $y_total /= 2;
    $denominator /= 2;

    $y_total = ($y_total == -1 ? '-'
                : sprintf "%+d", $y_total);
    if ($denominator == 1) {
      printf "  %d %s i,\n", $x_total, $y_total;
    } else {
      printf "  \\tfrac{%d %s i}{%d},\n", $x_total, $y_total, $denominator;
    }
  }
  print "  % end generated\n";
}
exit 0;
