#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::ComplexMinus 117; # v.117 for level_to_n_range()
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 5;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $ComplexFour = 0;
{
  # complex base i-1

  my $path = Math::PlanePath::ComplexMinus->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);

  print "  \\begin{scope}[font=\\tiny]\n";

  # NSEW squares
  my @ComplexFour_n_list;
  {
    my $is_visited = sub {
      my ($x,$y) = @_;
      my $n = $path->xy_to_n($x,$y);
      return (defined $n && $n <= $n_hi);
    };

    $Printwrap = 4;
    printwrap("\\fill[my gray] foreach \\p in {");
    $Printwrap += 2;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      if ($is_visited->($x+1,$y)
          && $is_visited->($x-1,$y)
          && $is_visited->($x,$y+1)
          && $is_visited->($x,$y-1)) {
        $x = 2*$x - 1;
        $y = 2*$y - 2;
        printwrap("($x,$y),");
        $ComplexFour++;
        push @ComplexFour_n_list, $n;
      }
    }
    printwrap("} ");
    printwrap("{ \\p +(\\MyGap,\\MyGap) rectangle +(2-\\MyGap,2-\\MyGap) };\n");
  }

  {
    my $segments = MyLineSegments->new;
    my @dir4_to_dx = (1,0,-1,0);
    my @dir4_to_dy = (0,1,0,-1);

    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      foreach my $dir4 (0 .. 3) {
        my $dx = $dir4_to_dx[$dir4];
        my $dy = $dir4_to_dy[$dir4];
        my $n_other = $path->xy_to_n($x+$dx,$y+$dy);
        $segments->add(2*$x+$dx - $dy,  2*$y+$dy - $dx - 1,
                       2*$x+$dx + $dy,  2*$y+$dy + $dx - 1);
      }
    }

    {
      $Printwrap = 4;
      printwrap("\\draw ");
      $Printwrap += 2;
      $segments->printwrap;
      printwrap(";\n");
    }
  }

  {
    # numbers
    $Printwrap = 4;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      $x = 2*$x;
      $y = 2*$y - 1;
      printwrap(" \\node at ($x,$y) {$n};");
    }
    printwrap_newline();
  }

  print "  \\end{scope}\n";
  print "  \\newcommand{\\MyComplexFour}{$ComplexFour}\n";
  print "  \\newcommand{\\MyComplexFourN}{",join(',',@ComplexFour_n_list),"}\n";
  print "% GP-Test  ComplexFour($level) == $ComplexFour\n";
  print "% GP-DEFINE  gen_complex_minus_NSEW_vector = [",join(',',@ComplexFour_n_list),"];\n";
  print "% GP-DEFINE  gen_complex_minus_NSEW_k = $level;\n";
}

{
  # twindragon

  my $path = Math::PlanePath::DragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

  print "  \\begin{scope}[my right pos]\n";
  print "    \\coordinate (end) at ($x_hi,$y_hi);\n";

  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi-1) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    $segments->add($x1,$y1, $x2,$y2);
    $segments->add($x_hi - $x1, $y_hi - $y1,
                   $x_hi - $x2, $y_hi - $y2);
  }

  $Printwrap = 4;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");

  my $xy_to_twindragon_n_count = sub {
    my ($x,$y) = @_;
    my @n_list = ($path->xy_to_n_list($x, $y),
                  $path->xy_to_n_list($x_hi-$x, $y_hi-$y));
    @n_list = grep {$_ < $n_hi} @n_list;
    return scalar(@n_list);
  };

  # # double-visited points
  # {
  #   my $count_doubles = 0;
  #   $Printwrap = 4;
  #   printwrap("\\path foreach \\p in {");
  #   $Printwrap += 2;
  #   foreach my $n ($n_lo .. $n_hi) {
  #     my ($x,$y) = $path->n_to_xy($n);
  #     if ($xy_to_twindragon_n_count->($x,$y) == 2) {
  #       printwrap("($x,$y),");
  #     }
  #   }
  #   printwrap("} ");
  #   printwrap("{ \\p node[my dot] };\n");
  #   print "% GP-Test  TD($level) == $count_doubles\n";
  # }

  # four-double squares
  {
    my $count = 0;
    $Printwrap = 4;
    printwrap("\\fill[my twindragon gray] foreach \\p in {");
    $Printwrap += 2;

    my %seen;
    foreach my $reverse (0, 1) {
      foreach my $n ($n_lo .. $n_hi-1) {
        my ($x,$y) = $path->n_to_xy($n);
        if ($reverse) {
          ($x,$y) = ($x_hi-$x, $y_hi-$y);
        }
        next if (($x+$y) % 2) == 1;
        next if $seen{"$x,$y"}++;

        if ($xy_to_twindragon_n_count->($x,$y) == 2
            && $xy_to_twindragon_n_count->($x+1,$y) == 2
            && $xy_to_twindragon_n_count->($x,$y+1) == 2
            && $xy_to_twindragon_n_count->($x+1,$y+1) == 2) {
          printwrap("($x,$y),");
          $count++;
        }
      }
    }
    printwrap("} ");
    printwrap("{ \\p +(\\MyTwindragonGap,\\MyTwindragonGap)");
    $Printwrap += 5;
    printwrap("rectangle +(1-\\MyTwindragonGap,1-\\MyTwindragonGap) };\n");
    $count == $ComplexFour or die;
  }

  $Printwrap = 2;
  print "  \\end{scope}\n";
}


print "  % end generated\n";
exit 0;
