#!/usr/bin/perl -w

# Copyright 2015 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'sum';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon 'xy_rotate_plus90','xy_rotate_minus90';

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 8;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

my $var = $FindBin::Script;
$var =~ s/\.pl$//;
$var =~ tr/-/_/;

my @count;
foreach my $k (0 .. $max_level) {
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);

  my $path_xyxy_is_traversed_either = sub {
    my ($x1,$y1, $x2,$y2) = @_;
    my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
    return (defined $n && $n >= $n_lo && $n < $n_hi);
  };
  my $path_n_is_right_boundary = sub {
    my ($n) = @_;
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    my $dx = $x2 - $x1;
    my $dy = $y2 - $y1;
    ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
    my $x1s = $x1 + $dx;
    my $y1s = $y1 + $dy;
    my $x2s = $x2 + $dx;
    my $y2s = $y2 + $dy;
    my $right = (! $path_xyxy_is_traversed_either->($x1,$y1, $x1s,$y1s)
                 || ! $path_xyxy_is_traversed_either->($x2,$y2, $x2s,$y2s)
                 || ! $path_xyxy_is_traversed_either->($x1s,$y1s, $x2s,$y2s));
    ### $right
    return $right;
  };
  my $path_n_is_left_boundary = sub {
    my ($n) = @_;
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    my $dx = $x2 - $x1;
    my $dy = $y2 - $y1;
    ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to side
    my $x1s = $x1 + $dx;
    my $y1s = $y1 + $dy;
    my $x2s = $x2 + $dx;
    my $y2s = $y2 + $dy;
    ### left consider: "n=$n  $x1,$y1 to $x2,$y2   and $x1s,$y1s to $x2s,$y2s  within $n_lo to n_hi=$n_hi"
    my $left = (! $path_xyxy_is_traversed_either->($x1,$y1, $x1s,$y1s)
                || ! $path_xyxy_is_traversed_either->($x2,$y2, $x2s,$y2s)
                || ! $path_xyxy_is_traversed_either->($x1s,$y1s, $x2s,$y2s));
    ### $left
    return $left;
  };

  my $prev_degree = 1;  # at N=0
  my @edges = (undef, undef, 0,0,0);  # 0,1,  1+1, 1+2, 2+2
  foreach my $n (1 .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    my @n_list = $path->xy_to_n_list($x,$y);
    @n_list = grep {$_ <= $n_hi} @n_list;
    my $degree = 2*scalar(@n_list) - ($n == $n_hi);
    my $type = join('', (sort {$a<=>$b} $degree, $prev_degree));
    $count[$k]->{$type}++;

    if ($degree == 4 && $prev_degree == 4) {
      if ($path_n_is_left_boundary->($n-1)
          || $path_n_is_right_boundary->($n-1)) {
        $count[$k]->{'44b'}++;
      } else {
        $count[$k]->{'44e'}++;
      }
    }

    $prev_degree = $degree;
  }
}

foreach my $type ('11','12','22','24','44','44b','44e') {
  print "% GP-DEFINE  ${var}_${type}_samples = [";
  my $join = '';
  foreach my $k (0 .. $#count) {
    my $value = delete $count[$k]->{$type} || 0;
    print $join,$value;
    $join = ',';
  }
  print "];\n";
}

foreach my $k (0 .. $#count) {
  my $href = $count[$k];
  if (%$href) {
    print "Oops, k=$k leftover keys: ",join(' ',keys %$href),"\n";
    exit 1;
  }
}

print "% end generated\n";
exit 0;
