#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyDragon
  '$hullsaround_segs_outer','$hullsaround_segs_inner','$hullsaround_segs_all',
  'xy_to_node_name', '@dir4_to_dx','@dir4_to_dy';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

my @segs = ("0,0,1",
            "0,0,2",
            "0,0,3",

            "1,1,0",
            "1,1,3",

            # "1,-1,0",
            "1,-1,1",
            "1,-1,2",
            "1,-1,3",

            "-1,-1,0",
            "-1,-1,1",
            "2,0,2",
           );

my %seen;
foreach my $seg ("0,0,0", @segs) {
  my ($x,$y,$dir) = split /,/, $seg;
  foreach (0, 1) {
    my $name = xy_to_node_name($x,$y);
    unless ($seen{$name}++) {
      my $blank = ($x==0 && $y==0) || ($x==1 && $y==0) ? '' : ' blank';
      print "  \\node ($name) at ($x,$y) [my$blank dot];\n";
    }
    $x += $dir4_to_dx[$dir];
    $y += $dir4_to_dy[$dir];
  }
}

foreach my $seg (@segs) {
  my ($x,$y,$dir) = split /,/, $seg;
  my $from = xy_to_node_name($x,$y);
  my $to   = xy_to_node_name($x+$dir4_to_dx[$dir], $y+$dir4_to_dy[$dir]);
  my $angle = 90*$dir;
  print "  \\draw[->] ($from)--($to);\n";
  print "  \\pic at ($x,$y) [transform shape,rotate=$angle] {my hull pic};\n";
}

print "  % end generated\n";
exit 0;
