#!/usr/bin/perl -w

# Copyright 2014, 2015 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve;

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon 'xy_rotate_minus90';

# uncomment this to run the ### lines
# use Smart::Comments;


my $pred_count = 16;
my $at_count = 15;

my $path = Math::PlanePath::DragonCurve->new;

sub n_is_right_boundary {
  my ($n) = @_;
  ### path_n_is_right_boundary(): $n
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! defined $path->xyxy_to_n_either($x1,$y1, $x1s,$y1s)
          || ! defined $path->xyxy_to_n_either($x2,$y2, $x2s,$y2s)
          || ! defined $path->xyxy_to_n_either($x1s,$y1s, $x2s,$y2s));
}

{
  print "% generated by tools/$FindBin::Script\n";
  print "% GP-VECTOR  Rpred_samples\n";
  print "  &= ";
  foreach my $n (0 .. $pred_count-1) {
    print n_is_right_boundary($n) ? "1," : "0,";
  }
  print " \\dotsc\n";
  print "% GP-END\n";
  print "% end generated\n";
  print "\n";
}

{
  print "% generated by tools/$FindBin::Script\n";
  print "% GP-VECTOR  Rpred_at_samples\n";
  print "  &= ";
  my $count = 0;
  for (my $n = 0; $count < $at_count; $n++) {
    if (n_is_right_boundary($n)) {
      print "$n,";
      $count++;
    }
  }
  print " \\dotsc\n";
  print "% GP-END\n";
  print "% end generated\n";
  print "\n";
}

{
  print "% generated by tools/$FindBin::Script\n";
  print "% GP-VECTOR  Rpred_non_at_samples\n";
  print "  &= ";
  my $count = 0;
  for (my $n = 0; $count < $at_count; $n++) {
    if (! n_is_right_boundary($n)) {
      print "$n,";
      $count++;
    }
  }
  print " \\dotsc\n";
  print "% GP-END\n";
  print "% end generated\n";
  print "\n";
}

exit 0;
