#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use POSIX 'ceil';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyLineSegments;
use MyPrintwrap ':all';
use MyDragon 'xy_rotate_minus90';


my $level = 3;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

sub transform {
  my ($x,$y) = @_;
  foreach (1 .. ceil($level/2)) {
    ($x,$y) = xy_rotate_minus90($x,$y);  # to left side
  }
  return ($x,$y);
}

$Printwrap = 2;
printwrap("\\node at (0,0) [my dot,name=start] {};\n");
{
  my ($x,$y) = transform($x_hi,$y_hi);
  printwrap("\\node at ($x,$y) [my dot,name=end] {};\n");
}

foreach my $n ($n_lo .. $n_hi-1) {
  foreach my $reverse (0, 1) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    if ($n%2 == 1) {
      ($x1,$y1, $x2,$y2) = ($x2,$y2, $x1,$y1);
    }
    if ($reverse) {
      ($x1,$y1) = ($x_hi - $x1, $y_hi - $y1);
      ($x2,$y2) = ($x_hi - $x2, $y_hi - $y2);
    }
    ($x1,$y1) = transform($x1,$y1);
    ($x2,$y2) = transform($x2,$y2);
    printwrap(" \\draw[->] ($x1,$y1)--($x2,$y2);");
  }
}
printwrap_newline();

print "  % end generated\n";
exit 0;
